from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1
soundThreshold = 50
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setLED(1, 100)  # Turn on the light
bird.setLED(1, 0)  # Turn off the light
sleep(1)

# Exercise 1 - modified (the sleep() times inside the loop need to be short.
# Otherwise, you may need to clap a long time.)
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setLED(1, 100)  # Blink
    sleep(0.1)
    bird.setLED(1, 0)
    sleep(0.1)

sleep(1)

# Exercise 2
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setLED(1, bird.getDial(1))  # Set brightness with dial

sleep(1)

# Exercise 2 - modified
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setRotationServo(1, bird.getDial(1))  # Set speed with dial

sleep(1)

# Exercise 3
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setPositionServo(1, 1.8 * bird.getDial(1))  # Set angle with dial

sleep(1)

# Exercise 4
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setTriLED(1, bird.getDial(1), 0, 100 - bird.getDial(1))  # Set color with dial

sleep(1)

# Exercise 5
while bird.getSound(2) < soundThreshold:  # While it is quiet
    bird.setRotationServo(1, 2 * bird.getDial(1) - 100)  # Set speed with dial

sleep(1)

# Extra Challenge
dialThreshold = 50
while bird.getDial(1) < dialThreshold:  # While the dial value is low
    bird.setTriLED(1, bird.getSound(2), 0, 100 - bird.getSound(2))  # Set color with sound sensor
bird.stopAll()



from birdbrain import Hummingbird
from time import sleep

hummingbird = Hummingbird('A')

for i in range(0, 10):
    hummingbird.led(1, 100)
    sleep(0.1)

    hummingbird.led(1, 0)
    sleep(0.1)

hummingbird.stop_all()



# This file is here for historical reasons and backward compatibility.
# Must use "from birdbrain" instead of "from BirdBrain" when importing

from birdbrain.constant import Constant
from birdbrain.finch import Finch
from birdbrain.hummingbird import Hummingbird
from birdbrain.microbit import Microbit
from birdbrain.tasks import Tasks

__all__ = [
    "Constant",
    "Finch",
    "Hummingbird",
    "Microbit",
    "Tasks",
]



from birdbrain.device import Device
from birdbrain.exception import Exception
from birdbrain.microbit_input import MicrobitInput
from birdbrain.microbit_output import MicrobitOutput
from birdbrain.request import Request


class Microbit(Device):
    def __init__(self, device='A', raise_exception_if_no_connection=True):
        self.device_object = Microbit.connect(device, raise_exception_if_no_connection)

    def display(self, list):
        return MicrobitOutput.display(self.state, self.device, list)

    def clear_display(self):
        return MicrobitOutput.clear_display(self.state, self.device)

    def point(self, x, y, value):
        return MicrobitOutput.point(self.state, self.device, x, y, value)

    def print(self, message):
        return MicrobitOutput.print(self.state, self.device, message)

    def play_note(self, note, beats):
        return MicrobitOutput.play_note(self.device, note, beats)

    def beep(self):
        return MicrobitOutput.play_note(self.device, 80, 0.333)

    def acceleration(self):
        return MicrobitInput.acceleration(self.device)

    def compass(self):
        return MicrobitInput.compass(self.device)

    def magnetometer(self):
        return MicrobitInput.magnetometer(self.device)

    def button(self, button):
        return MicrobitInput.button(self.device, button)

    def sound(self, port=None):
        return MicrobitInput.sound(self.device)

    def temperature(self):
        return MicrobitInput.temperature(self.device)

    def is_shaking(self):
        return MicrobitInput.is_shaking(self.device)

    def orientation(self):
        return MicrobitInput.orientation(self.device)

    def stop_all(self):
        Request.stop_all(self.device)

    getAcceleration = acceleration
    getButton = button
    getCompass = compass
    setDisplay = display
    isShaking = is_shaking
    getMagnetometer = magnetometer
    getOrientation = orientation
    playNote = play_note
    setPoint = point
    getSound = sound
    stopAll = stop_all
    getTemperature = temperature



import time
import urllib.request

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.utility import Utility


class Request:
    @classmethod
    def uri(self, *args):
        return "http://127.0.0.1:30061/" + Utility.flatten_string(args)

    @classmethod
    def is_not_connected_response(self, response):
        return response.lower() == "not connected"

    @classmethod
    def response(self, *args):
        if "false" in args:
            return False

        try:
            if Constant.BIRDBRAIN_TEST:
                print("Test: URI", self.uri(*args))

            response_request = urllib.request.urlopen(self.uri(*args))
        except (ConnectionError, urllib.error.URLError, urllib.error.HTTPError):
            raise (Exception("Error: Request to device failed"))

        response = response_request.read().decode('utf-8').lower()

        if Constant.BIRDBRAIN_TEST:
            print("Test: response", response)

        if self.is_not_connected_response(response):
            raise (Exception("Error: The device is not connected"))

        time.sleep(0.01)  # hack to prevent http requests from overloading the BlueBird Connector

        return response

    @classmethod
    def response_status(self, *args):
        return Request.request_status(Request.response(args))

    @classmethod
    def is_connected(self, device):
        try:
            self.response('hummingbird', 'in', 'orientation', 'Shake', device)
        except Exception:
            return False

        return True

    @classmethod
    def is_not_connected(self, device):
        return not self.is_connected(device)

    @classmethod
    def stop_all(self, device):
        return self.request_status(self.response('hummingbird', 'out', 'stopall', device))

    @classmethod
    def request_status(self, status):
        if Constant.BIRDBRAIN_TEST:
            print("Test: request status is", status)

        if status is None:
            return None

        if status == 'true':
            return True
        if status == 'led set':
            return True
        if status == 'triled set':
            return True
        if status == 'servo set':
            return True
        if status == 'buzzer set':
            return True
        if status == 'symbol set':
            return True
        if status == 'print set':
            return True
        if status == 'all stopped':
            return True

        if status == 'finch moved':
            return True
        if status == 'finch turned':
            return True
        if status == 'finch wheels started':
            return True
        if status == 'finch wheels stopped':
            return True
        if status == 'finch encoders reset':
            return True

        if status == 'false':
            return False
        if status == 'not connected':
            return False
        if status == 'invalid orientation':
            return False
        if status == 'invalid port':
            return False

        return None

    @classmethod
    def calculate_angle(self, intensity):
        return int(int(intensity) * 255 / 180)

    @classmethod
    def calculate_intensity(self, intensity):
        return int(int(Utility.bounds(intensity, 0, 100)) * 255 / 100)

    @classmethod
    def calculate_speed(self, speed):
        if int(speed) in range(-10, 10):
            return 255

        # QUESTION: why this calculation instead of normal mapping to 0..255 (and 255 means stop)
        # return ((int(speed) * 23 / 100) + 122)

        if int(speed) < 0:
            return int(119 - (-int(speed) / 100 * 45))
        else:
            return int((int(speed) / 100 * 25) + 121)

    @classmethod
    def calculate_left_or_right(self, direction):
        if direction == Constant.LEFT:
            return 'Left'
        if direction == Constant.RIGHT:
            return 'Right'

        return 'None'

    @classmethod
    def validate(self, validate, valid_range, validate_message):
        if not str(validate) in valid_range:
            raise Exception(validate_message)

        return True

    @classmethod
    def validate_port(self, port, valid_range, allow_all=False):
        if allow_all and str(port) == 'all':
            return True

        return Request.validate(port, valid_range, f"Port {str(port)} out of range.")

    @classmethod
    def sensor_response(self, device, sensor, other=None, options={}):
        if other is False:
            return False  # for invalid directions

        factor = options["factor"] if "factor" in options else Constant.DEFAULT_FACTOR
        min_response = options["min_response"] if "min_response" in options else Constant.DEFAULT_UNLIMITED_MIN_RESPONSE
        max_response = options["max_response"] if "max_response" in options else Constant.DEFAULT_UNLIMITED_MAX_RESPONSE
        type_method = options["type_method"] if "type_method" in options else Constant.DEFAULT_TYPE_METHOD

        request = ['hummingbird', 'in', sensor]
        if other is not None:
            request.append(other)
        request.append(device)

        response = float(Request.response(request)) * factor

        response = round(Utility.decimal_bounds(response, min_response, max_response), 3)

        if type_method == 'int':
            return int(response)

        return response

    @classmethod
    def xyz_response(self, device, sensor, type_method='int'):
        x = round(float(Request.response('hummingbird', 'in', sensor, 'X', device)), 3)
        y = round(float(Request.response('hummingbird', 'in', sensor, 'Y', device)), 3)
        z = round(float(Request.response('hummingbird', 'in', sensor, 'Z', device)), 3)

        if type_method == 'int':
            return [int(x), int(y), int(z)]
        else:
            return [float(x), float(y), float(z)]

    @classmethod
    def tri_led_response(self, device, port, r_intensity, g_intensity, b_intensity, valid_range, allow_all=False):
        """Set TriLED  of a certain port requested to a valid intensity."""
        self.validate_port(port, valid_range, allow_all)

        calc_r = Request.calculate_intensity(r_intensity)
        calc_g = Request.calculate_intensity(g_intensity)
        calc_b = Request.calculate_intensity(b_intensity)

        return Request.response_status('hummingbird', 'out', 'triled', port, calc_r, calc_g, calc_b, device)

    @classmethod
    def orientation_response(self, device, sensor, orientations, orientation_results, orientation_in_between):
        for index, target_orientation in enumerate(orientations):
            response = self.response("hummingbird", "in", sensor, target_orientation, device)

            if response == "true":
                return orientation_results[index]

        return orientation_in_between



from birdbrain.utility import Utility


def test_is_none_or_empty():
    assert Utility.is_none_or_empty(None)
    assert Utility.is_none_or_empty('')
    assert not Utility.is_none_or_empty('something')


def test_bounds():
    assert Utility.bounds(10, 0, 100) == 10
    assert Utility.bounds(10, -100, 100) == 10
    assert Utility.bounds(-10, -100, 100) == -10
    assert Utility.bounds(-100, -100, 100) == -100
    assert Utility.bounds(100, -100, 100) == 100

    assert Utility.bounds(101, -100, 100) == 100
    assert Utility.bounds(-101, -100, 100) == -100
    assert Utility.bounds(999999, -100, 100) == 100
    assert Utility.bounds(-999999, -100, 100) == -100

    assert Utility.bounds(10.0, 0.0, 100.0) == 10.0
    assert Utility.bounds(10.0, -100.0, 100.0) == 10.0
    assert Utility.bounds(-10.0, -100.0, 100.0) == -10.0
    assert Utility.bounds(-100.0, -100.0, 100.0) == -100.0
    assert Utility.bounds(100.0, -100.0, 100.0) == 100.0

    assert Utility.bounds(101.0, -100.0, 100.0) == 100.0
    assert Utility.bounds(-101.0, -100.0, 100.0) == -100.0
    assert Utility.bounds(999999.0, -100.0, 100.0) == 100.0
    assert Utility.bounds(-999999.0, -100.0, 100.0) == -100.0

    assert Utility.bounds(str(10), str(0), str(100)) == 10
    assert Utility.bounds(str(10), str(-100), str(100)) == 10
    assert Utility.bounds(str(-10), str(-100), str(100)) == -10
    assert Utility.bounds(str(-100), str(-100), str(100)) == -100
    assert Utility.bounds(str(100), str(-100), str(100)) == 100

    assert Utility.bounds(str(101), str(-100), str(100)) == 100
    assert Utility.bounds(str(-101), str(-100), str(100)) == -100
    assert Utility.bounds(str(999999), str(-100), str(100)) == 100
    assert Utility.bounds(str(-999999), str(-100), str(100)) == -100

def test_decimal_bounds():
    assert Utility.decimal_bounds(10, 0, 100) == 10
    assert Utility.decimal_bounds(10, -100, 100) == 10
    assert Utility.decimal_bounds(-10, -100, 100) == -10
    assert Utility.decimal_bounds(-100, -100, 100) == -100
    assert Utility.decimal_bounds(100, -100, 100) == 100

    assert Utility.decimal_bounds(101, -100, 100) == 100
    assert Utility.decimal_bounds(-101, -100, 100) == -100
    assert Utility.decimal_bounds(999999, -100, 100) == 100
    assert Utility.decimal_bounds(-999999, -100, 100) == -100

    assert Utility.decimal_bounds(10.0, 0.0, 100.0) == 10.0
    assert Utility.decimal_bounds(10.0, -100.0, 100.0) == 10.0
    assert Utility.decimal_bounds(-10.0, -100.0, 100.0) == -10.0
    assert Utility.decimal_bounds(-100.0, -100.0, 100.0) == -100.0
    assert Utility.decimal_bounds(100.0, -100.0, 100.0) == 100.0

    assert Utility.decimal_bounds(101.0, -100.0, 100.0) == 100.0
    assert Utility.decimal_bounds(-101.0, -100.0, 100.0) == -100.0
    assert Utility.decimal_bounds(999999.0, -100.0, 100.0) == 100.0
    assert Utility.decimal_bounds(-999999.0, -100.0, 100.0) == -100.0

    assert Utility.decimal_bounds(str(10), str(0), str(100)) == 10
    assert Utility.decimal_bounds(str(10), str(-100), str(100)) == 10
    assert Utility.decimal_bounds(str(-10), str(-100), str(100)) == -10
    assert Utility.decimal_bounds(str(-100), str(-100), str(100)) == -100
    assert Utility.decimal_bounds(str(100), str(-100), str(100)) == 100

    assert Utility.decimal_bounds(str(101), str(-100), str(100)) == 100
    assert Utility.decimal_bounds(str(-101), str(-100), str(100)) == -100
    assert Utility.decimal_bounds(str(999999), str(-100), str(100)) == 100
    assert Utility.decimal_bounds(str(-999999), str(-100), str(100)) == -100

def test_flatten():
    flattened = Utility.flatten_string(["something", "1", ["A", "B"], "2", "else", 99, [99]])

    assert flattened == "something/1/A/B/2/else/99/99"


def test_flatten_tuple():
    flattened = Utility.flatten_string(("something", "1", ["A", "B"], "2", "else", 99, [99]))

    assert flattened == "something/1/A/B/2/else/99/99"



from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.request import Request
from birdbrain.state import State
from time import sleep


class Device:
    def __init__(self, device="A", raise_exception_if_no_connection=True):
        self.state = State()
        self.device = Device.remap_device(device)
        self.connected = False

    @classmethod
    def connect(self, device="A", raise_exception_if_no_connection=True):
        device_object = Device(device)

        self.state = device_object.state
        self.device = device_object.device
        self.connected = device_object.connected

        if device is None:
            raise Exception("Missing device name")
        if device not in Constant.VALID_DEVICES:
            raise Exception("Invalid device name: " + device)

        self.connected = device_object.connect_device()

        if raise_exception_if_no_connection and not device_object.connected:
            raise Exception("No connection: " + device)

        return device_object

    def is_connected(self):
        """Determine if the device is connected"""

        return self.connected

    def __is_device(self, operator):
        response = Request.response("hummingbird", "in", operator, "static", self.device)

        return response == 'true'

    def is_microbit(self):
        """Determine if the device is a Microbit"""

        # allow hummingbird/finch to be seen as microbit
        # return self.__is_device("isMicrobit")
        return True

    def is_hummingbird(self):
        """Determine if the device is a hummingbird."""
        return self.__is_device("isHummingbird")

    def is_finch(self):
        """Determine if the device is a Finch"""

        return self.__is_device("isFinch")

    def remap_device(device):
        return device

    def connect_device(self):
        self.connected = Request.is_connected(self.device)

        return self.connected

    def stop_all(self):
        Request.stop_all(self.device)

    def set_cache(self, name, value):
        return self.state.set(name, value)

    def get_cache(self, name):
        return self.state.get(name)

    def sleep(self, seconds):
        sleep(seconds)

    isConnectionValid = is_connected
    isFinch = is_finch
    isHummingbird = is_hummingbird
    isMicrobit = is_microbit



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1
bird.playNote(60, 1)  # C
sleep(1)
bird.playNote(62, 0.5)  # D
sleep(0.5)
bird.playNote(64, 2)  # E
sleep(2)

# Exercise 2
# Example 1
bird.playNote(72, 1)
sleep(1)
bird.setLED(1, 100)
sleep(1)
bird.setLED(1, 0)
sleep(1)

# Example 2
bird.playNote(72, 1)
bird.setLED(1, 100)
sleep(1)
bird.setLED(1, 0)
sleep(1)

# Exercise 3
for i in [60, 65, 69, 72, 60]:  # For each note in the list
    bird.playNote(i, 0.5)  # Play the note
    sleep(0.5)

# Exercise 4 - Answers will vary

# Extra Challenge
beats = [1, 0.5, 1, 2, 0.5]
notes = [60, 65, 69, 72, 60]
for i in range(5):  # For each of the five entries in the lists
    bird.playNote(notes[i], beats[i])  # Play the note with right number of beats
    sleep(beats[i])

bird.stopAll()



import pytest

from birdbrain.exception import Exception
from birdbrain.hummingbird import Hummingbird

def test_exception():
    exception = Exception("MESSAGE")

    assert str(exception) == "MESSAGE"

def test_exception_stop_all():
    hummingbird = Hummingbird('A')

    exception = Exception("STOP", hummingbird)

    assert str(exception) == "STOP"



import pytest

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.request import Request


def test_request_uri():
    uri = Request.uri(["in", "1", "2", "3", "4", ["99", 99], "something"])

    assert uri == "http://127.0.0.1:30061/in/1/2/3/4/99/99/something"


def test_connected():
    assert Request.is_connected("A")


def test_not_connected():
    assert not Request.is_connected("C")


def test_not_connected_connected():
    assert not Request.is_not_connected("A")


def test_not_connected_not_connected():
    assert Request.is_not_connected("C")


def test_response_with_false_arg():
    assert not Request.response("1", "false", "2")


def test_response():
    assert Request.response("hummingbird", "in", "orientation", "Shake", "A")


def test_response_status():
    assert not Request.response_status("hummingbird", "in", "orientation", "Shake", "A")


def test_response_no_connection():
    with pytest.raises(Exception) as e:
        Request.response("hummingbird", "in", "orientation", "Shake", "C")

    assert e.value.message == "Error: The device is not connected"


def test_request_status():
    assert Request.request_status("all stopped")
    assert Request.request_status(None) is None


def test_stop_all():
    response = Request.stop_all("A")

    assert response


def test_disconnect():
    with pytest.raises(Exception) as e:
        Request.stop_all("C")

    assert e.value.message == "Error: The device is not connected"


def test_xyz_response_no_connection():
    with pytest.raises(Exception):
        Request.xyz_response("C", "Accelerometer")


def test_xyz_response():
    xyz = Request.xyz_response("A", "Accelerometer", "float")

    assert isinstance(xyz, list)
    assert len(xyz) == 3


def test_calculate_speed():
    assert Request.calculate_speed(0) == 255
    assert Request.calculate_speed(9) == 255
    assert Request.calculate_speed(100) == 146.0
    assert Request.calculate_speed(-100) == 74.0

    assert Request.calculate_speed("0") == 255
    assert Request.calculate_speed("9") == 255
    assert Request.calculate_speed("100") == 146.0
    assert Request.calculate_speed("-100") == 74.0


def test_calculate_left_or_right():
    assert Request.calculate_left_or_right('L') == 'Left'
    assert Request.calculate_left_or_right('R') == 'Right'
    assert Request.calculate_left_or_right('BAD') == 'None'


def test_validate_port():
    assert Request.validate_port(1, Constant.VALID_LED_PORTS)
    assert Request.validate_port(2, Constant.VALID_LED_PORTS)
    assert Request.validate_port(3, Constant.VALID_LED_PORTS)
    assert Request.validate_port("1", Constant.VALID_LED_PORTS)

    with pytest.raises(Exception):
        Request.validate_port(4, Constant.VALID_LED_PORTS)
    with pytest.raises(Exception):
        Request.validate_port(-1, Constant.VALID_LED_PORTS)
    with pytest.raises(Exception):
        Request.validate_port("4", Constant.VALID_LED_PORTS)


def test_debugging():
    Constant.BIRDBRAIN_TEST = True

    assert Request.response("hummingbird", "in", "orientation", "Shake", "A")
    assert Request.request_status("false") is False
    assert Request.request_status("not connected") is False
    assert Request.request_status("invalid orientation") is False
    assert Request.request_status("invalid port") is False
    assert Request.request_status("nonsense") is None


def test_sensor_response():
    assert Request.sensor_response(None, None, False) is False


def test_orientation_response(mocker):
    mocker.patch.object(Request, "response", return_value="false")

    assert Request.orientation_response(None, None, "unknown", [], "in between") == "in between"



import pytest

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.finch_input import FinchInput
from birdbrain.finch_output import FinchOutput
from birdbrain.request import Request


def test_is_moving():
    assert FinchOutput.move("B", Constant.FORWARD, 7, 5, False)
    assert FinchInput.is_moving("B")

    FinchOutput.wait("B")

    assert FinchOutput.move("B", Constant.BACKWARD, 7, 5, True)

    assert Request.stop_all("B")

    assert not FinchInput.is_moving("B")


def test_light():
    response = FinchInput.light("B", "L")

    assert 0 <= response <= 100
    assert isinstance(response, int)

    response = FinchInput.light("B", "R")

    assert 0 <= response <= 100
    assert isinstance(response, int)

    with pytest.raises(Exception) as e:
        FinchInput.light("B", "BAD")
    assert e.value.message == "Error: Request to device failed"

    with pytest.raises(Exception) as e:
        FinchInput.light("B", None)
    assert e.value.message == "Error: Request to device failed"


def test_distance():
    response = FinchInput.distance("B")

    assert 0 <= response <= 298
    assert isinstance(response, int)


def test_line():
    response = FinchInput.line("B", "L")

    assert 0 <= response <= 100
    assert isinstance(response, int)

    response = FinchInput.line("B", "R")

    assert 0 <= response <= 100
    assert isinstance(response, int)

    with pytest.raises(Exception) as e:
        FinchInput.line("B", "BAD")
    assert e.value.message == "Error: Request to device failed"

    with pytest.raises(Exception) as e:
        FinchInput.line("B", None)
    assert e.value.message == "Error: Request to device failed"


def test_encoder():
    response = FinchInput.encoder("B", "L")

    assert -100.0 <= response <= 100.0
    assert isinstance(response, float)

    response = FinchInput.encoder("B", "R")

    assert -100.0 <= response <= 100.0
    assert isinstance(response, float)

    with pytest.raises(Exception) as e:
        FinchInput.encoder("B", "BAD")
    assert e.value.message == "Error: Request to device failed"

    with pytest.raises(Exception) as e:
        FinchInput.encoder("B", None)
    assert e.value.message == "Error: Request to device failed"


def test_acceleration():
    response = FinchInput.acceleration("B")

    assert -100.0 <= response[0] <= 100.0
    assert -100.0 <= response[1] <= 100.0
    assert -100.0 <= response[2] <= 100.0

    assert isinstance(response[0], float)
    assert isinstance(response[1], float)
    assert isinstance(response[2], float)


def test_compass():
    response = FinchInput.compass("B")

    assert 0 <= response <= 359
    assert isinstance(response, int)


def test_magnetometer():
    response = FinchInput.magnetometer("B")

    assert -180.0 <= response[0] <= 180.0
    assert -180.0 <= response[1] <= 180.0
    assert -180.0 <= response[2] <= 180.0

    assert isinstance(response[0], int)
    assert isinstance(response[1], int)
    assert isinstance(response[2], int)


def test_orientation():
    response = FinchInput.orientation("B")

    some_position = False
    for orientation in Constant.FINCH_ORIENTATION_RESULTS:
        some_position = some_position or (orientation == response)

    assert some_position



from birdbrain import Hummingbird, Microbit
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object
bit = Microbit('B')  # Declare Microbit object

# Exercise 1
while not (bit.getOrientation() == "Screen down"):
    if bit.getButton('A'):
        bird.setTriLED(1, 0, 0, 100)
    elif bit.getButton('B'):
        bird.setTriLED(1, 0, 0, 0)

# Exercise 2
while not (bit.getButton('A')):  # While button A isn't pressed
    if bit.getOrientation() == "Tilt left":
        bird.playNote(60, 1)
        bird.setPositionServo(1, 0)
        bird.setTriLED(1, 100, 0, 0)
    elif bit.getOrientation() == "Tilt right":
        bird.playNote(62, 1)
        bird.setPositionServo(1, 60)
        bird.setTriLED(1, 0, 100, 0)
    elif bit.getOrientation() == "Logo down":
        bird.playNote(64, 1)
        bird.setPositionServo(1, 120)
        bird.setTriLED(1, 0, 0, 100)
    elif bit.getOrientation() == "Logo up":
        bird.playNote(65, 1)
        bird.setPositionServo(1, 180)
        bird.setTriLED(1, 100, 0, 100)
    elif bit.getOrientation() == "Screen up":
        bird.playNote(67, 1)
        bird.setPositionServo(1, 90)
        bird.setTriLED(1, 100, 100, 0)
    elif bit.getOrientation() == "Screen down":
        bird.playNote(69, 1)
        bird.setPositionServo(1, 30)
        bird.setTriLED(1, 0, 100, 100)

sleep(1)

# Exercise 3
while not (bit.getButton('A')):  # While button A isn't pressed
    if bit.getCompass() > 75 and bit.getCompass() < 105:  # Direction between 75 and 105
        bit.print('E')
        bird.setTriLED(1, 100, 0, 0)
        bird.setRotationServo(1, 100)
    elif bit.getCompass() > 165 and bit.getCompass() < 195:  # Direction between 165 and 195
        bit.print('S')
        bird.setTriLED(1, 0, 100, 0)
        bird.setRotationServo(1, -100)
    elif bit.getCompass() > 255 and bit.getCompass() < 285:  # Direction between 255 and 285
        bit.print('W')
        bird.setTriLED(1, 0, 0, 100)
        bird.setRotationServo(1, 50)
    elif bit.getCompass() > 345 or bit.getCompass() < 15:  # Direction between 255 and 285
        bit.print('N')
        bird.setTriLED(1, 100, 0, 100)
        bird.setRotationServo(1, -50)
    else:
        bird.setDisplay([0] * 25)  # Clear screen
        bird.setTriLED(1, 0, 0, 0)
        bird.setRotationServo(1, 0)

# Exercise 4
while not (bird.getButton('A')):
    if bird.getDistance(1) < 30:
        bit.print("CLOSE")
        sleep(3)
    else:
        bit.print("FAR")
        sleep(2)

bird.stopAll()
bit.stopAll()



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1
bird.setTriLED(1, 100, 0, 0)  # Turn LED Red
sleep(1)
bird.setTriLED(1, 0, 100, 0)  # Turn LED Green
sleep(1)
bird.setTriLED(1, 0, 0, 100)  # Turn LED Blue
sleep(1)

# Exercise 2
bird.setTriLED(1, 100, 0, 100)  # Turn LED Purple
sleep(1)
bird.setTriLED(1, 0, 100, 100)  # Turn LED Aqua
sleep(1)
bird.setTriLED(1, 100, 100, 0)  # Turn LED Yellow
sleep(1)

# Exercise 3
for i in range(5):
    bird.setTriLED(1, 0, 100, 0)  # LED Green
    sleep(0.5)
    bird.setTriLED(1, 0, 0, 0)  # LED Off
    sleep(0.5)

bird.setTriLED(1, 100, 0, 0)  # LED Red
sleep(1)

# Exercise 4
for i in range(6):
    bird.setTriLED(1, 100, 0, 0)  # LED Red
    sleep(0.5)
    bird.setTriLED(1, 0, 0, 0)  # LED Off
    sleep(0.5)

for i in range(3):
    bird.setTriLED(1, 0, 0, 100)  # LED Blue
    sleep(0.5)
    bird.setTriLED(1, 0, 0, 0)  # LED Off
    sleep(0.5)

for i in range(5):
    bird.setTriLED(1, 0, 100, 0)  # LED Green
    sleep(0.5)
    bird.setTriLED(1, 0, 0, 0)  # LED Off
    sleep(0.5)

# Exercise 5
for i in range(5):
    bird.setTriLED(1, 100, 0, 100)  # LED Purple
    sleep(1)
    bird.setTriLED(1, 0, 100, 0)  # LED Green
    sleep(1)
for i in range(5):
    bird.setTriLED(1, 0, 100, 100)  # LED Aqua
    sleep(0.25)
    bird.setTriLED(1, 100, 0, 0)  # LED Red
    sleep(0.25)

# Extra Challenge
for i in range(100):
    bird.setTriLED(1, i, 0, 100 - i)
    sleep(0.1)

bird.stopAll()  # Turn everything off



import asyncio


class Tasks:
    def __init__(self):
        self.method_list = []
        self.task_list = []
        self.results = {}

    def result(self, method_name):
        return self.results[method_name] if method_name in self.results else None

    def create_task(self, method):
        self.method_list.append(method)

    def run(self):
        asyncio.run(self.runner())

    async def wait(self):
        while True:
            running_task_count = 0

            for task in self.task_list:
                try:
                    self.results[task.get_coro().__name__] = task.result()
                except asyncio.exceptions.InvalidStateError:
                    pass

                if not task.done():
                    running_task_count += 1

            if running_task_count == 0:
                break

            await self.yield_task()

    async def runner(self):
        for method_name in self.method_list:
            self.task_list.append(asyncio.create_task(method_name))

        await self.wait()

    @classmethod
    async def yield_task(self, yield_time=0.0):
        await asyncio.sleep(yield_time)



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

while bird.getButton('A'):  # while button A is pressed
    bird.setTriLED(1, 100, 0, 100)
bird.setTriLED(1, 0, 0, 0)

sleep(1)

# Exercise 1
while not (bird.getButton('A')):  # while button A isn't pressed
    bird.setTriLED(1, 100, 0, 100)  # purple
bird.setTriLED(1, 0, 0, 0)  # off

# Exercise 2
while not (bird.getButton('B')):  # while button B isn't pressed
    if bird.getButton('A'):
        bird.setTriLED(1, 0, 100, 0)
    else:
        bird.setTriLED(1, 0, 0, 0)

# micro:bit V2 challenge
while not (bird.getButton('B')):  # while button B isn't pressed
    if bird.getButton("Logo"):
        bird.setPositionServo(1, 180)
    else:
        bird.setPositionServo(1, 0)

# Exercise 3
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getOrientation() == "Tilt left":  # If micro:bit tilted left
        bird.setPositionServo(1, 0)  # Set angle to 0°
    else:  # Otherwise
        bird.setPositionServo(1, 90)  # Set angle to 90°

# Exercise 3 - modified
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getOrientation() == "Tilt right":  # If micro:bit tilted right
        bird.setPositionServo(1, 180)  # Set angle to 180°
    else:  # Otherwise
        bird.setPositionServo(1, 90)  # Set angle to 90°

# Exercise 4
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getOrientation() == "Tilt left":  # If micro:bit tilted left
        bird.setPositionServo(1, 0)  # Set angle to 0°
    elif bird.getOrientation() == "Tilt right":  # If micro:bit tilted right
        bird.setPositionServo(1, 180)  # Set angle to 180
    else:  # Otherwise
        bird.setPositionServo(1, 90)  # Set angle to 0°

# Exercise 4 - modified
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getOrientation() == "Tilt left":  # If micro:bit tilted left
        bird.setPositionServo(1, 0)  # Set angle to 0°
    elif bird.getOrientation() == "Tilt right":  # If micro:bit tilted right
        bird.setPositionServo(1, 180)  # Set angle to 180
    elif bird.getOrientation() == "Logo down":  # If tilted logo down
        bird.setPositionServo(1, 30)  # Set angle to 30°
    else:  # Otherwise
        bird.setPositionServo(1, 90)  # Set angle to 0°

# Exercise 5 - I chose not to use an else so that I didn't play a note for "In between"
# but students may choose to include an else
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getOrientation() == "Tilt left":
        bird.playNote(60, 1)
    elif bird.getOrientation() == "Tilt right":
        bird.playNote(62, 1)
    elif bird.getOrientation() == "Logo down":
        bird.playNote(64, 1)
    elif bird.getOrientation() == "Logo up":
        bird.playNote(65, 1)
    elif bird.getOrientation() == "Screen up":
        bird.playNote(67, 1)
    elif bird.getOrientation() == "Screen down":
        bird.playNote(69, 1)
    sleep(1)

# Exercise 6
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getOrientation() == "Tilt left":
        bird.playNote(60, 1)
        bird.setPositionServo(1, 0)
        bird.setTriLED(1, 100, 0, 0)
    elif bird.getOrientation() == "Tilt right":
        bird.playNote(62, 1)
        bird.setPositionServo(1, 60)
        bird.setTriLED(1, 0, 100, 0)
    elif bird.getOrientation() == "Logo down":
        bird.playNote(64, 1)
        bird.setPositionServo(1, 120)
        bird.setTriLED(1, 0, 0, 100)
    elif bird.getOrientation() == "Logo up":
        bird.playNote(65, 1)
        bird.setPositionServo(1, 180)
        bird.setTriLED(1, 100, 0, 100)
    elif bird.getOrientation() == "Screen up":
        bird.playNote(67, 1)
        bird.setPositionServo(1, 90)
        bird.setTriLED(1, 100, 100, 0)
    elif bird.getOrientation() == "Screen down":
        bird.playNote(69, 1)
        bird.setPositionServo(1, 30)
        bird.setTriLED(1, 0, 100, 100)
    sleep(1)

# Extra Challenge
while not (bird.getButton('B')):  # while button B isn't pressed
    accelList = bird.getAcceleration()  # Store x, y, and z values of acceleration
    bird.setTriLED(
        1, 10 * abs(accelList[0]), 10 * abs(accelList[1]), 10 * abs(accelList[2])
    )  # Each acceleration value determines one RGB intensity

bird.stopAll()

# micro:bit V2 challenge
if bird.getTemperature() < 18:
    bird.playNote(48, 1)
    print("Brrr, I'm cold.")
elif bird.getTemperature() > 25:
    bird.playNote(72, 1)
    print("I'm burning up!")
else:
    bird.playNote(60, 1)
    print("I'm just right.")
sleep(1)
bird.stopAll()



import asyncio
import random

from birdbrain.hummingbird import Hummingbird
from birdbrain.tasks import Tasks


async def random_blinker(hummingbird):
    for i in range(35):
        hummingbird.tri_led(1, random.randint(0, 100), random.randint(0, 100), random.randint(0, 100))

        await Tasks.yield_task()

    return "random_blinker"  # return is optional


async def blue_blinker(hummingbird):
    for i in range(35):
        hummingbird.tri_led(1, 0, 0, 100)

        await Tasks.yield_task()


hummingbird = Hummingbird('A')

tasks = Tasks()

tasks.create_task(random_blinker(hummingbird))
tasks.create_task(blue_blinker(hummingbird))

tasks.run()

random_blinker_result = tasks.result("random_blinker")

hummingbird.tri_led(1, 0, 0, 0)



import time

from birdbrain.hummingbird import Hummingbird
from birdbrain.hummingbird_output import HummingbirdOutput


def test_led():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.led(hummingbird.device, 1, 50)
    time.sleep(0.15)

    HummingbirdOutput.led(hummingbird.device, 1, "0")


def test_tri_led():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.tri_led(hummingbird.device, 1, 50, "50", 0)
    time.sleep(0.15)

    HummingbirdOutput.tri_led(hummingbird.device, 1, 0, 0, 0)


def test_position_servo():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.position_servo(hummingbird.device, 1, 20)
    time.sleep(0.15)

    HummingbirdOutput.position_servo(hummingbird.device, 1, 160)
    time.sleep(0.15)


def test_rotation_servo():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.rotation_servo(hummingbird.device, 2, 25)
    time.sleep(0.15)

    HummingbirdOutput.rotation_servo(hummingbird.device, "2", "-25")
    time.sleep(0.15)

    HummingbirdOutput.rotation_servo(hummingbird.device, 2, 0)



from birdbrain.exception import Exception
from birdbrain.finch_input import FinchInput
from birdbrain.finch_output import FinchOutput
from birdbrain.microbit import Microbit


class Finch(Microbit):
    """The Finch class includes the control of the outputs and inputs present
    in the Finch robot. When creating an instance, specify which robot by the
    device letter used in the BlueBirdConnector device list (A, B, or C)."""

    def __init__(self, device='A', raise_exception_if_no_connection=True):
        self.device_object = Finch.connect(device, raise_exception_if_no_connection)

        if not self.device_object.is_finch():
            raise Exception("Error: Device " + device + " is not a Finch")

    def is_moving(self):
        return FinchInput.is_moving(self.device)

    def beak(self, r_intensity, g_intensity, b_intensity):
        return FinchOutput.beak(self.device, r_intensity, g_intensity, b_intensity)

    def tail(self, port, r_intensity, g_intensity, b_intensity):
        return FinchOutput.tail(self.device, port, r_intensity, g_intensity, b_intensity)

    def move(self, direction, distance, speed, wait_to_finish_movement=True):
        return FinchOutput.move(self.device, direction, distance, speed, wait_to_finish_movement)

    def turn(self, direction, angle, speed, wait_to_finish_movement=True):
        return FinchOutput.turn(self.device, direction, angle, speed, wait_to_finish_movement)

    def motors(self, left_speed, right_speed):
        return FinchOutput.motors(self.device, left_speed, right_speed)

    def wait(self, device):
        return FinchOutput.wait(self.device)

    def stop(self):
        return FinchOutput.stop(self.device)

    def reset_encoders(self):
        return FinchOutput.reset_encoders(self.device)

    def light(self, side):
        return FinchInput.light(self.device, side)

    def distance(self):
        return FinchInput.distance(self.device)

    def line(self, side):
        return FinchInput.line(self.device, side)

    def encoder(self, side):
        return FinchInput.encoder(self.device, side)

    def orientation(self):
        return FinchInput.orientation(self.device)

    def acceleration(self):
        return FinchInput.acceleration(self.device)

    def compass(self):
        return FinchInput.compass(self.device)

    def magnetometer(self):
        return FinchInput.magnetometer(self.device)

    getAcceleration = acceleration
    setBeak = beak
    getCompass = compass
    getDistance = distance
    getEncoder = encoder
    getLight = light
    getLine = line
    getMagnetometer = magnetometer
    setMotors = motors
    setMove = move
    getOrientation = orientation
    resetEncoders = reset_encoders
    setTail = tail
    setTurn = turn



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getCompass() > 75 and bird.getCompass() < 105:  # If both Booleans are true
        bird.print('E')  # Print E
    else:
        bird.print('')  # Print nothing

sleep(1)

# Exercise 1 - Modified
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getCompass() > 75 and bird.getCompass() < 105:  # Direction between 75 and 105
        bird.print('E')
    elif bird.getCompass() > 165 and bird.getCompass() < 195:  # Direction between 165 and 195
        bird.print('S')
    else:
        bird.print('')  # Clear screen


# Exercise 2
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getCompass() > 75 and bird.getCompass() < 105:  # Direction between 75 and 105
        bird.print('E')
    elif bird.getCompass() > 165 and bird.getCompass() < 195:  # Direction between 165 and 195
        bird.print('S')
    elif bird.getCompass() > 255 and bird.getCompass() < 285:  # Direction between 255 and 285
        bird.print('W')
    else:
        bird.print('')  # Clear screen

# Exercise 3
while not (bird.getButton('A')):
    print(bird.getCompass())
    # While button A isn't pressed
    if bird.getCompass() > 75 and bird.getCompass() < 105:  # Direction between 75 and 105
        bird.print('E')
    elif bird.getCompass() > 165 and bird.getCompass() < 195:  # Direction between 165 and 195
        bird.print('S')
    elif bird.getCompass() > 255 and bird.getCompass() < 285:  # Direction between 255 and 285
        bird.print('W')
    elif bird.getCompass() > 345 or bird.getCompass() < 15:  # Direction between 255 and 285
        bird.print('N')
    else:
        bird.print('')  # Clear screen

# Exercise 4
while not (bird.getButton('A')):  # While button A isn't pressed
    if bird.getCompass() > 75 and bird.getCompass() < 105:  # Direction between 75 and 105
        bird.print('E')
        bird.setTriLED(1, 100, 0, 0)
        bird.setRotationServo(1, 100)
    elif bird.getCompass() > 165 and bird.getCompass() < 195:  # Direction between 165 and 195
        bird.print('S')
        bird.setTriLED(1, 0, 100, 0)
        bird.setRotationServo(1, -100)
    elif bird.getCompass() > 255 and bird.getCompass() < 285:  # Direction between 255 and 285
        bird.print('W')
        bird.setTriLED(1, 0, 0, 100)
        bird.setRotationServo(1, 50)
    elif bird.getCompass() > 345 or bird.getCompass() < 15:  # Direction between 255 and 285
        bird.print('N')
        bird.setTriLED(1, 100, 0, 100)
        bird.setRotationServo(1, -50)
    else:
        bird.print('')  # Clear screen
        bird.setTriLED(1, 0, 0, 0)
        bird.setRotationServo(1, 0)


# Going Further - if you tape the Hummingbird Bit on top of the rotation servo, this code
# rotates the Bit until the compass is pointing south
while not (bird.getButton('A')):
    if bird.getCompass() > 190:
        bird.setRotationServo(1, 20)
    elif bird.getCompass() < 170:
        bird.setRotationServo(1, -30)
    else:
        bird.setRotationServo(1, 0)

bird.stopAll()



from birdbrain.constant import Constant
from birdbrain.request import Request


class MicrobitInput(Request):
    @classmethod
    def acceleration(self, device, sensor="Accelerometer"):
        """Gives the acceleration of X,Y,Z in m/sec2."""

        return self.xyz_response(device, sensor, "float")

    @classmethod
    def compass(self, device, sensor='Compass'):
        """Returns values 0-359 indicating the orentation of the Earth's
        magnetic field."""

        sensor_options = {}
        sensor_options['min_response'] = Constant.DEFAULT_DEGREES_MIN_RESPONSE
        sensor_options['max_response'] = Constant.DEFAULT_DEGREES_MAX_RESPONSE

        compass_option = None if sensor == 'Compass' else 'static'

        return self.sensor_response(device, sensor, compass_option, sensor_options)

    @classmethod
    def magnetometer(self, device, sensor="Magnetometer"):
        """Return the values of X,Y,Z of a magnetommeter."""
        return self.xyz_response(device, sensor, "int")

    @classmethod
    def button(self, device, button):
        """Return the status of the button asked. Specify button 'A', 'B', or
        'Logo'. Logo available for V2 micro:bit only."""

        return self.request_status(self.response('hummingbird', 'in', 'button', button.capitalize(), device))

    @classmethod
    def sound(self, device):
        """Return the current sound level as an integer between 1 and 100.
        Available for V2 micro:bit only."""

        response = self.response('hummingbird', 'in', "V2sensor", "Sound", device)

        if response == 'micro:bit v2 required':
            return 0

        return int(response)

    @classmethod
    def temperature(self, device):
        """Return the current temperature as an integer in degrees Celcius.
        Available for V2 micro:bit only."""

        response = self.response('hummingbird', 'in', "V2sensor", "Temperature", device)

        if response == 'micro:bit v2 required':
            return 0

        return int(response)

    @classmethod
    def is_shaking(self, device):
        """Return true if the device is shaking, false otherwise."""

        return self.request_status(self.response('hummingbird', 'in', 'orientation', 'Shake', device))

    @classmethod
    def orientation(self, device):
        """Return the orentation of the Microbit. Results found in Constant.HUMMINGBIRD_ORIENTATION_RESULTS"""
        return self.orientation_response(
            device,
            "orientation",
            Constant.HUMMINGBIRD_ORIENTATIONS,
            Constant.HUMMINGBIRD_ORIENTATION_RESULTS,
            Constant.HUMMINGBIRD_ORIENTATION_IN_BETWEEN,
        )



from birdbrain.constant import Constant
from birdbrain.microbit_input import MicrobitInput
from birdbrain.request import Request


class FinchInput(Request):
    @classmethod
    def is_moving(self, device):
        return Request.request_status(Request.response('hummingbird', 'in', 'finchIsMoving', 'static', device))

    @classmethod
    def light(self, device, side):
        """Read the value of the right or left light sensor ('R' or 'L')."""

        return self.sensor_response(device, 'Light', Request.calculate_left_or_right(side))

    @classmethod
    def distance(self, device):
        """Read the value of the distance sensor"""

        distance_options = {}
        distance_options['factor'] = Constant.DISTANCE_FACTOR  # was 0.0919
        distance_options['min_response'] = Constant.DEFAULT_UNLIMITED_MIN_RESPONSE
        distance_options['max_response'] = Constant.DEFAULT_UNLIMITED_MAX_RESPONSE

        return self.sensor_response(device, 'Distance', 'static', distance_options)

    @classmethod
    def line(self, device, side):
        """Read the value of the right or left line sensor ('R' or 'L').
        Returns brightness as a value 0-100 where a larger number
        represents more reflected light."""

        return self.sensor_response(device, 'Line', Request.calculate_left_or_right(side))

    @classmethod
    def encoder(self, device, side):
        """Read the value of the right or left encoder ('R' or 'L').
        Values are returned in rotations."""

        sensor_options = {}
        sensor_options['min_response'] = float(Constant.DEFAULT_UNLIMITED_MIN_RESPONSE)
        sensor_options['max_response'] = float(Constant.DEFAULT_UNLIMITED_MAX_RESPONSE)
        sensor_options['type_method'] = 'float'

        return round(self.sensor_response(device, 'Encoder', Request.calculate_left_or_right(side), sensor_options), 2)

    @classmethod
    def orientation(self, device):
        """Return the orentation of the Finch. Results found in Constant.FINCH_ORIENTATION_RESULTS"""
        return self.orientation_response(
            device,
            "finchOrientation",
            Constant.FINCH_ORIENTATIONS,
            Constant.FINCH_ORIENTATION_RESULTS,
            Constant.FINCH_ORIENTATION_IN_BETWEEN,
        )

    # The following methods override those within the Microbit
    # class to return values within the Finch reference frame.
    @classmethod
    def acceleration(self, device):
        """Gives the acceleration of X,Y,Z in m/sec2, relative
        to the Finch's position."""

        return MicrobitInput.acceleration(device, "finchAccel")

    @classmethod
    def compass(self, device):
        """Returns values 0-359 indicating the orentation of the Earth's
        magnetic field, relative to the Finch's position."""

        return MicrobitInput.compass(device, "finchCompass")

    @classmethod
    def magnetometer(self, device):
        """Return the values of X,Y,Z of a magnetommeter, relative to the Finch's position."""

        return MicrobitInput.magnetometer(device, "finchMag")



from birdbrain.constant import Constant
from birdbrain.microbit_input import MicrobitInput
from birdbrain.request import Request


class HummingbirdInput(Request):
    @classmethod
    def acceleration(self, device):
        """Gives the acceleration of X,Y,Z in m/sec2, relative
        to the Finch's position."""

        return MicrobitInput.acceleration(device)

    @classmethod
    def compass(self, device):
        """Returns values 0-359 indicating the orentation of the Earth's
        magnetic field, relative to the Finch's position."""

        return MicrobitInput.compass(device)

    @classmethod
    def magnetometer(self, device):
        """Return the values of X,Y,Z of a magnetommeter, relative to the Finch's position."""

        return MicrobitInput.magnetometer(device)

    @classmethod
    def orientation(self, device):
        """Return the orentation of the Hummingbird. Results found in Constant.HUMMINGBIRD_ORIENTATION_RESULTS"""

        return MicrobitInput.orientation(device)

    @classmethod
    def sensor(self, device, port):
        """Read the value of the sensor attached to a certain port."""

        sensor_options = {}
        sensor_options['min_response'] = Constant.DEFAULT_UNLIMITED_MIN_RESPONSE
        sensor_options['max_response'] = Constant.DEFAULT_UNLIMITED_MAX_RESPONSE
        sensor_options['type_method'] = 'float'

        return self.sensor_response(device, 'sensor', port, sensor_options)

    @classmethod
    def light(self, device, port):
        """Read the value of the light sensor attached to a certain port."""

        sensor_options = {}
        sensor_options['factor'] = Constant.LIGHT_FACTOR
        sensor_options['min_response'] = Constant.DEFAULT_MIN_RESPONSE
        sensor_options['max_response'] = Constant.DEFAULT_MAX_RESPONSE

        return self.sensor_response(device, 'sensor', port, sensor_options)

    @classmethod
    def sound(self, device, port):
        """Read the value of the sound sensor attached to a certain port."""

        port = str(port).lower()

        if port == "microbit" or port == "micro:bit":
            return MicrobitInput.sound(device)

        sensor_options = {}
        sensor_options['factor'] = Constant.SOUND_FACTOR
        sensor_options['min_response'] = Constant.DEFAULT_MIN_RESPONSE
        sensor_options['max_response'] = Constant.DEFAULT_MAX_RESPONSE

        return self.sensor_response(device, 'sensor', port, sensor_options)

    @classmethod
    def distance(self, device, port):
        """Read the value of the distance sensor attached to a certain port."""

        sensor_options = {}
        sensor_options['factor'] = Constant.DISTANCE_FACTOR
        sensor_options['min_response'] = Constant.DEFAULT_MIN_RESPONSE
        sensor_options['max_response'] = Constant.DEFAULT_UNLIMITED_MAX_RESPONSE

        return self.sensor_response(device, 'sensor', port, sensor_options)

    @classmethod
    def dial(self, device, port):
        """Read the value of the dial attached to a certain port."""

        sensor_options = {}
        sensor_options['factor'] = Constant.DIAL_FACTOR
        sensor_options['min_response'] = Constant.DEFAULT_MIN_RESPONSE
        sensor_options['max_response'] = Constant.DEFAULT_MAX_RESPONSE

        return self.sensor_response(device, 'sensor', port, sensor_options)

    @classmethod
    def voltage(self, device, port):
        """Read the value of  the dial attached to a certain port."""

        sensor_options = {}
        sensor_options['factor'] = Constant.VOLTAGE_FACTOR
        sensor_options['min_response'] = Constant.VOLTAGE_MIN
        sensor_options['max_response'] = Constant.VOLTAGE_MAX
        sensor_options['type_method'] = 'float'

        return self.sensor_response(device, 'sensor', port, sensor_options)



class Utility:
    @classmethod
    def is_none_or_empty(self, s):
        if s is None or s == "":
            return True
        else:
            return False

    @classmethod
    def bounds(self, input, input_min, input_max, pass_through_input=None):
        # if pass_through_input is not None and (input == pass_through_input): return int(input)

        if int(input) < int(input_min):
            return int(input_min)
        if int(input) > int(input_max):
            return int(input_max)

        return int(input)

    @classmethod
    def decimal_bounds(self, input, input_min, input_max, pass_through_input=None):
        # if pass_through_input is not None and (input == pass_through_input): return int(input)

        if float(input) < float(input_min):
            return float(input_min)
        if float(input) > float(input_max):
            return float(input_max)

        return float(input)

    @classmethod
    def flatten_string(self, original_list, divider="/"):
        if isinstance(original_list[0], list):
            original_list = original_list[0]

        original_list = [item for item in original_list]

        s = ""
        for element in list(original_list):
            if isinstance(element, str):
                s += str(element) + divider
            elif isinstance(element, int):
                s += str(element) + divider
            else:
                for sub_element in element:
                    s += str(sub_element) + divider

        return s[:-1]



import pytest
import random

from birdbrain.device import Device
from birdbrain.exception import Exception
from time import sleep

def test_none_device():
    with pytest.raises(Exception) as e:
        Device.connect(None)
    assert e.value.message == "Missing device name"


def test_bad_device():
    with pytest.raises(Exception) as e:
        Device.connect("Z")
    assert e.value.message == "Invalid device name: Z"


def test_stop_all():
    hummingbird = Device.connect()

    hummingbird.stop_all()


def test_default_connect():
    hummingbird = Device.connect()

    assert hummingbird.connected
    assert hummingbird.device == 'A'


def test_connect():
    hummingbird = Device.connect("A")

    assert hummingbird.connected
    assert hummingbird.device == 'A'


def test_connect_to_disconnected_device():
    with pytest.raises(Exception) as e:
        Device.connect("C", True)
    assert e.value.message == "No connection: C"


def test_connect_to_disconnected_device_no_exception():
    hummingbird = Device.connect("C", False)

    assert not hummingbird.connected
    assert hummingbird.device == 'C'


def test_connect_to_disconnected_device_with_exception():
    with pytest.raises(Exception) as e:
        Device.connect("C", True)
    assert e.value.message == "No connection: C"


def test_is_hummingbird():
    hummingbird = Device.connect("A")

    assert hummingbird.is_hummingbird


def test_is_finch():
    hummingbird = Device.connect("A")

    assert not hummingbird.is_finch()


def test_cache():
    hummingbird = Device.connect("A")

    assert hummingbird.get_cache("something_name") is None

    assert hummingbird.set_cache("something_name", "something") == "something"
    assert hummingbird.get_cache("something_name") == "something"

    assert "something_name" in hummingbird.state.cache

    assert hummingbird.set_cache("something_name", None) is None

    assert "something_name" not in hummingbird.state.cache

    assert hummingbird.get_cache("something_name") is None

    assert hummingbird.set_cache("set_not_in_the_cache", None) is None

def test_sleep():
    hummingbird = Device.connect('A')

    hummingbird.sleep(0.1)



class State:
    def __init__(self):
        self.cache = {}
        self.display_map = State.microbit_empty_display_map()

    def display_map_clear(self):
        self.display_map = State.microbit_empty_display_map()

    def set_list(self, list):
        self.display_map = list

    def set_pixel(self, x, y, value):
        self.display_map[((x * 5) + y - 6)] = value

    def display_map_normalize(self):
        return ["true" if ((pixel == 1) or (pixel is True)) else "false" for pixel in self.display_map]

    def display_map_as_string(self, list=None):
        if list is not None:
            self.set_list(list)

        return "/".join(self.display_map_normalize())

    def set(self, name, value):
        if value is None:
            if name in self.cache:
                self.cache.pop(name)
        else:
            self.cache[name] = value

        return value

    def get(self, name):
        if name in self.cache:
            return self.cache[name]
        else:
            return None

    @classmethod
    def microbit_empty_display_map(self):
        return [0] * 25



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

print(bird.getDistance(1))  # Print sensor value

# Exercise 1
threshold = 20
if bird.getDistance(1) < threshold:
    bird.setTriLED(1, 100, 0, 0)
else:
    bird.setTriLED(1, 0, 100, 0)
sleep(2)

# Exercise 2
threshold = 30
if bird.getDistance(1) > threshold:
    bird.print("FAR")
else:
    bird.print("NEAR")
sleep(3)

# Exercise 3
threshold = 20
while bird.getDistance(1) < threshold:  # As long as distance less than threshold
    bird.setTriLED(1, 100, 0, 0)  # Set tri-color LED red

bird.setTriLED(1, 0, 100, 0)  # Set tri-color LED green
sleep(1)

# Exercise 7 - This code works the same as Exercise 3, but there are no statements
# in the while loop. It just pauses the program until an object is close to the distance sensor
threshold = 20
bird.setTriLED(1, 100, 0, 0)  # Set tri-color LED red
while bird.getDistance(1) < threshold:  # As long as distance less than threshold
    pass  # Do nothing

bird.setTriLED(1, 0, 100, 0)  # Set tri-color LED green
sleep(1)

# Exercise 4
while bird.getDistance(1) > threshold:  # As long as distance > threshold
    bird.setTriLED(1, 0, 100, 0)  # Set tri-color LED green
bird.setTriLED(1, 100, 0, 0)  # Set tri-color LED red
bird.playNote(60, 1)
sleep(1)

# Exercise 5
while bird.getDistance(1) < threshold:
    bird.setPositionServo(1, 60)
    sleep(0.5)
    bird.setPositionServo(1, 0)
    sleep(0.5)

# Exercise 6 - You may have to hold your hand in front of the distance
# sensor up to 6 seconds to make the LED stop blinking. The Boolean statement
# is only checked at the top of the loop. Once the light turns blue, it will
# be 6 seconds before the Boolean statement is checked again.
while bird.getDistance(1) > threshold:
    bird.setTriLED(1, 0, 0, 100)
    sleep(3)
    bird.setTriLED(1, 100, 0, 100)
    sleep(3)

# Extra Challenge - This code controls a servo and LED with a distance sensor
# until the light sensor is covered.
lightThreshold = 10
distThreshold = 20
while bird.getLight(2) > lightThreshold:
    if bird.getDistance(1) < distThreshold:
        bird.setTriLED(1, 0, 100, 100)
        bird.setPositionServo(1, 90)
    else:
        bird.setTriLED(1, 0, 0, 0)
        bird.setPositionServo(1, 0)

bird.stopAll()



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

bird.print("Hi")  # Print Hi on micro:bit display
sleep(1)

bird.print("Bambi")  # Exercise 1: Name
sleep(3)

bird.setDisplay([1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1])  # Display a pattern
sleep(1)

bird.setDisplay([0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0])  # Exercise 2: Display a smiley face
sleep(1)

for i in range(10):  # Exercise 3: Animation
    bird.setDisplay([0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0])
    sleep(0.25)
    bird.setDisplay([0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0])
    sleep(0.25)

# Exercise 4
bird.setDisplay([0] * 25)  # Clear the display

bird.setPoint(1, 1, 1)  # Turn on all the corners
bird.setPoint(1, 5, 1)
bird.setPoint(5, 1, 1)
bird.setPoint(5, 5, 1)
sleep(1)

# Exercise 5
bird.setDisplay([1] * 25)  # Turn on all the LEDS

bird.setPoint(1, 1, 0)  # Turn off all the corners
bird.setPoint(1, 5, 0)
bird.setPoint(5, 1, 0)
bird.setPoint(5, 5, 0)
sleep(1)

# Exercise 6
bird.setDisplay([0] * 25)  # Clear the display
for i in range(5):
    bird.setPoint(i + 1, i + 1, 1)
    sleep(0.5)

# Extra Challenge
bird.setDisplay([0] * 25)  # Clear the display
for i in range(5):
    for j in range(5):
        bird.setPoint(i + 1, j + 1, 1)
        sleep(0.5)
for i in range(5):  # Simple example - students might do much cooler things!
    for j in range(5):
        bird.setPoint(i + 1, j + 1, 0)
        sleep(0.5)

bird.stopAll()  # Turn everything off



class Exception(Exception):
    def __init__(self, message, birdbrain_device=None):
        if birdbrain_device is not None:
            birdbrain_device.stop_all()

        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return self.message



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

bird.setPositionServo(1, 90)

for i in range(5):
    bird.setPositionServo(1, 0)
    sleep(1)
    bird.setPositionServo(1, 180)
    sleep(1)

for i in range(180):
    bird.setPositionServo(1, i)
    sleep(0.1)
for i in range(180):
    bird.setPositionServo(1, 180 - i)
    sleep(0.1)

for i in range(100):
    bird.setPositionServo(1, 1.8 * i)
    bird.setLED(1, i)
    sleep(0.1)
for i in range(100):
    bird.setPositionServo(1, 180 - 1.8 * i)
    bird.setLED(1, 100 - i)
    sleep(0.1)


angle = input("What angle do you want? ")
bird.setPositionServo(1, int(angle))

bird.stopAll()



from birdbrain.tasks import Tasks


async def method_1(p):
    for i in range(40):
        print("method_1 running", p, i)

        await Tasks.yield_task()

    return "method_1_return"


async def method_2(p):
    for i in range(20):
        print("method_2 running", p, i)

        await Tasks.yield_task()

    return 2


async def method_3():
    for i in range(30):
        print("method_3 running", i)

        await Tasks.yield_task()


def test_tasks_with_wait():
    tasks = Tasks()

    tasks.create_task(method_1(999))
    tasks.create_task(method_2("text"))
    tasks.create_task(method_3())

    tasks.run()

    assert tasks.result("method_1") == "method_1_return"
    assert tasks.result("method_2") == 2
    assert tasks.result("method_3") is None
    assert tasks.result("unknown") is None
    assert tasks.result(None) is None



from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.request import Request
from birdbrain.state import State
from birdbrain.utility import Utility


class MicrobitOutput(Request):
    @classmethod
    def display(self, state, device, list):
        if len(list) != 25:
            raise Exception("Error: display() requires a list of length 25")

        return Request.response_status('hummingbird', 'out', 'symbol', device, state.display_map_as_string(list))

    @classmethod
    def clear_display(self, state, device):
        return self.display(state, device, State.microbit_empty_display_map())

    @classmethod
    def point(self, state, device, x, y, value):
        index = (x * 5) + y - 6

        try:
            state.display_map[index] = value
        except IndexError:
            raise Exception("Error: point out of range")

        return self.display(state, device, state.display_map)

    @classmethod
    def print(self, state, device, message):
        """Print the characters on the LED screen."""

        # clear internal representation of the display since it will be blank when the print ends
        self.clear_display(state, device)

        # if no message specified print as a blank
        if message is None or len(message) == 0:
            message = ' '

        # need to encode space for uri (used to be %20)
        message = message.replace(' ', '+')

        return Request.response_status('hummingbird', 'out', 'print', message, device)

    @classmethod
    def play_note(self, device, note, beats):
        """Make the buzzer play a note for certain number of beats. Note is the midi
        note number and should be specified as an integer from 32 to 135. Beats can be
        any number from 0 to 16. One beat corresponds to one second."""

        note = Utility.bounds(note, 32, 135)
        beats = int(Utility.decimal_bounds(beats, 0, 16) * Constant.BEATS_TEMPO_FACTOR)

        return Request.response_status('hummingbird', 'out', 'playnote', note, beats, device)



from pynput import keyboard

class Constant:
    BIRDBRAIN_TEST = False

    BACKWARD = 'B'
    BEATS_TEMPO = 60
    BEATS_TEMPO_FACTOR = 1000  # (60000 / TEMPO)
    DEFAULT_DEGREES_MAX_RESPONSE = 359
    DEFAULT_DEGREES_MIN_RESPONSE = 0
    DEFAULT_DEVICE = 'A'
    DEFAULT_FACTOR = 1.0
    DEFAULT_MAX_RESPONSE = 100.0
    DEFAULT_MIN_RESPONSE = 0.0
    DEFAULT_TYPE_METHOD = 'int'
    DEFAULT_UNLIMITED_MAX_RESPONSE = 1000000
    DEFAULT_UNLIMITED_MIN_RESPONSE = -1000000
    FORWARD = 'F'
    HUMMINGBIRD_ORIENTATIONS = ["Screen%20Up", "Screen%20Down", "Tilt%20Left", "Tilt%20Right", "Logo%20Up", "Logo%20Down"]
    HUMMINGBIRD_ORIENTATION_IN_BETWEEN = 'In between'
    HUMMINGBIRD_ORIENTATION_RESULTS = ["Screen up", "Screen down", "Tilt left", "Tilt right", "Logo up", "Logo down"]
    LEFT = 'L'
    MOVE_CHECK_MOVING_DELAY = 0.05
    MOVE_START_WAIT_SECONDS = 0.15
    MOVE_TIMEOUT_SECONDS = 60.0
    FINCH_ORIENTATIONS = ['Beak%20Up', 'Beak%20Down', 'Tilt%20Left', 'Tilt%20Right', 'Level', 'Upside%20Down']
    FINCH_ORIENTATION_IN_BETWEEN = 'In between'
    FINCH_ORIENTATION_RESULTS = ['Beak up', 'Beak down', 'Tilt left', 'Tilt right', 'Level', 'Upside down', 'In between']
    RESET_ENCODERS_DELAY = 0.2
    RIGHT = 'R'
    VALID_BEAK_PORTS = '1'
    VALID_BUTTONS = 'ABLogo'
    VALID_DEVICES = 'ABC'
    VALID_LED_PORTS = '123'
    VALID_MOVE_DIRECTION = 'FB'
    VALID_SENSOR_PORTS = '123'
    VALID_SERVO_PORTS = '1234'
    VALID_TAIL_PORTS = '2345all'
    VALID_TRI_LED_PORTS = '1234'
    VALID_TURN_DIRECTION = 'LR'
    VOLTAGE_MAX = 3.3
    VOLTAGE_MIN = 0.0

    BATTERY_FACTOR = 0.0406
    DIAL_FACTOR = 0.43478260869  # 100/230
    DISTANCE_FACTOR = 1.0  # 117/100 originally
    LIGHT_FACTOR = 0.39215686274  # 100/255
    SOUND_FACTOR = 0.78431372549  # 200/255
    VOLTAGE_FACTOR = 0.01294117647  # 3.3/255

    KEY_ALT = keyboard.Key.alt
    KEY_ALT_L = keyboard.Key.alt_l
    KEY_ALT_R = keyboard.Key.alt_r
    KEY_ALT_GR = keyboard.Key.alt_gr
    KEY_BACKSPACE = keyboard.Key.backspace
    KEY_CAPS_LOCK = keyboard.Key.caps_lock
    KEY_CMD = keyboard.Key.cmd
    KEY_CMD_L = keyboard.Key.cmd_l
    KEY_CMD_R = keyboard.Key.cmd_r
    KEY_CTRL = keyboard.Key.ctrl
    KEY_CTRL_L = keyboard.Key.ctrl_l
    KEY_CTRL_R = keyboard.Key.ctrl_r
    KEY_DELETE = keyboard.Key.delete
    KEY_DOWN = keyboard.Key.down
    KEY_END = keyboard.Key.end
    KEY_ENTER = keyboard.Key.enter
    KEY_ESC = keyboard.Key.esc
    KEY_F1 = keyboard.Key.f1
    KEY_F2 = keyboard.Key.f2
    KEY_F3 = keyboard.Key.f3
    KEY_F4 = keyboard.Key.f4
    KEY_F5 = keyboard.Key.f5
    KEY_F6 = keyboard.Key.f6
    KEY_F7 = keyboard.Key.f7
    KEY_F8 = keyboard.Key.f8
    KEY_F9 = keyboard.Key.f9
    KEY_F10 = keyboard.Key.f10
    KEY_F11 = keyboard.Key.f11
    KEY_F12 = keyboard.Key.f12
    KEY_F13 = keyboard.Key.f13
    KEY_F14 = keyboard.Key.f14
    KEY_F15 = keyboard.Key.f15
    KEY_F16 = keyboard.Key.f16
    KEY_F17 = keyboard.Key.f17
    KEY_F18 = keyboard.Key.f18
    KEY_F19 = keyboard.Key.f19
    KEY_F20 = keyboard.Key.f20
    KEY_HOME = keyboard.Key.home
    KEY_LEFT = keyboard.Key.left
    KEY_PAGE_DOWN = keyboard.Key.page_down
    KEY_PAGE_UP = keyboard.Key.page_up
    KEY_RIGHT = keyboard.Key.right
    KEY_SHIFT = keyboard.Key.shift
    KEY_SHIFT_L = keyboard.Key.shift_l
    KEY_SHIFT_R = keyboard.Key.shift_r
    KEY_SPACE = keyboard.Key.space
    KEY_TAB = keyboard.Key.tab
    KEY_UP = keyboard.Key.up
    KEY_MEDIA_PLAY_PAUSE = keyboard.Key.media_play_pause
    KEY_MEDIA_VOLUME_MUTE = keyboard.Key.media_volume_mute
    KEY_MEDIA_VOLUME_DOWN = keyboard.Key.media_volume_down
    KEY_MEDIA_VOLUME_UP = keyboard.Key.media_volume_up
    KEY_MEDIA_PREVIOUS = keyboard.Key.media_previous
    KEY_MEDIA_NEXT = keyboard.Key.media_next
    #KEY_INSERT = keyboard.Key.insert
    #KEY_MENU = keyboard.Key.menu
    #KEY_NUM_LOCK = keyboard.Key.num_lock
    #KEY_PAUSE = keyboard.Key.pause
    #KEY_PRINT_SCREEN = keyboard.Key.print_screen
    #KEY_SCROLL_LOCK = keyboard.Key.scroll_lock



from birdbrain.state import State


def test_state():
    state = State()

    for pixel in state.display_map:
        assert pixel == 0

    assert state.display_map[0] == 0
    assert state.display_map[18] == 0

    state.set_pixel(1, 1, 1)
    state.set_pixel(4, 4, 1)

    assert state.display_map[0] == 1
    assert state.display_map[18] == 1
    assert state.display_map[1] == 0
    assert state.display_map[19] == 0

    s = state.display_map_normalize()

    assert s[0] == "true"
    assert s[18] == "true"
    assert s[1] == "false"
    assert s[19] == "false"

    assert state.display_map_as_string() == (
        "true/false/false/false/false/false/false/false/false/false/"
        "false/false/false/false/false/false/false/false/true/false/false/false/false/false/false"
    )


def test_display_map_as_string_with_list():
    state = State()

    list = [0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0]

    assert state.display_map_as_string(list)[0:11] == "false/true/"


def test_state_using_true_and_false():
    state = State()

    state.set_pixel(1, 1, False)
    state.set_pixel(4, 4, True)

    s = state.display_map_normalize()

    assert s[0] == "false"
    assert s[18] == "true"

def test_display_map_clear():
    state = State()

    state.set_pixel(4, 4, True)
    assert state.display_map[18]

    state.display_map_clear()
    assert state.display_map[18] == 0

def test_cache():
    state = State()

    assert state.get("something_name") is None

    assert state.set("something_name", "something") == "something"
    assert state.get("something_name") == "something"

    assert "something_name" in state.cache

    assert state.set("something_name", None) is None

    assert "something_name" not in state.cache

    assert state.get("something_name") is None

    assert state.set("set_not_in_the_cache", None) is None



from birdbrain.exception import Exception
from birdbrain.hummingbird import Hummingbird
from birdbrain.microbit import Microbit

import pytest
import time


def test_connect_device_name_as_none():
    with pytest.raises(Exception) as e:
        Microbit(None)
    assert e.value.message == "Missing device name"


def test_connect_bad_device_name():
    with pytest.raises(Exception) as e:
        Microbit.connect('D')
    assert e.value.message == "Invalid device name: D"


def test_connect_valid_device_name():
    microbit = Microbit.connect("A")

    assert microbit.device == "A"


def test_is():
    microbit = Microbit.connect("A")

    assert microbit.is_connected()
    assert microbit.is_microbit()
    assert microbit.is_hummingbird()
    assert not microbit.is_finch()

    assert microbit.isConnectionValid()
    assert microbit.isMicrobit()
    assert microbit.isHummingbird()
    assert not microbit.isFinch()


def test_display_with_alias():
    hummingbird = Hummingbird("A")

    assert hummingbird.display([1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1])

    time.sleep(0.15)

    assert hummingbird.setDisplay([0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0])

    time.sleep(0.15)

    hummingbird.stop_all()


def test_display_wrong_size():
    with pytest.raises(Exception) as e:
        hummingbird = Hummingbird("A")

        hummingbird.display([0, 1])
    assert e.value.message == "Error: display() requires a list of length 25"


def test_point_and_clear_display_with_alias():
    hummingbird = Hummingbird("A")

    for i in range(2):
        assert hummingbird.point(2, 2, 1)
        assert hummingbird.point(2, 4, 1)
        assert hummingbird.point(4, 2, 1)
        assert hummingbird.setPoint(4, 4, 1)

        time.sleep(0.15)

        hummingbird.clear_display()


def test_point_true_or_false():
    hummingbird = Hummingbird("A")

    assert hummingbird.point(3, 3, True)

    time.sleep(0.15)

    assert hummingbird.point(3, 3, False)


def test_point_out_of_range():
    with pytest.raises(Exception) as e:
        hummingbird = Hummingbird("A")

        assert hummingbird.point(999, 1, 1)
    assert e.value.message == "Error: point out of range"


def test_print():
    hummingbird = Hummingbird("A")

    hummingbird.print("A")

    time.sleep(1)


def test_play_note_with_alias():
    hummingbird = Hummingbird("A")

    hummingbird.play_note(75, 0.5)

    time.sleep(0.25)

    hummingbird.playNote(40, 0.5)

    time.sleep(0.25)

    hummingbird.beep()


def test_stop_all():
    hummingbird = Hummingbird("A")

    hummingbird.stop_all()
    hummingbird.stopAll()



from birdbrain import Hummingbird
from time import sleep
import random  # import random number generator

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1
bird.setRotationServo(2, 100)  # Turn on at full speed
sleep(1)

# Exercise 2
bird.setRotationServo(2, -100)  # Clockwise quickly
sleep(2)
bird.setRotationServo(2, 25)  # Counterclockwise slowly
sleep(3)

# Exercise 3
for i in range(10):
    bird.setRotationServo(2, 100)
    sleep(0.5)
    bird.setRotationServo(2, 0)
    sleep(0.5)

# Exercise 4
for i in range(5):
    bird.setRotationServo(2, random.randint(0, 100))
    sleep(1)

# Exercise 5
for i in range(5):
    bird.setRotationServo(2, random.randint(-100, 100))
    sleep(1)

# Exercise 6
for i in range(5):
    bird.setTriLED(2, random.randint(0, 101), random.randint(0, 101), random.randint(0, 101))
    sleep(1)

# Extra Challenge
for i in range(5):
    bird.setTriLED(2, random.randint(0, 101), random.randint(0, 101), random.randint(0, 101))
    # sleep(random.random())     # Random number from 0-1
    sleep(5 * random.random())  # Random number from 0-5

bird.stopAll()



import pytest

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.microbit_input import MicrobitInput
from birdbrain.request import Request


def test_acceleration():
    response = MicrobitInput.acceleration("A", "Accelerometer")
    response = MicrobitInput.acceleration("A")

    assert -100.0 <= response[0] <= 100.0
    assert -100.0 <= response[1] <= 100.0
    assert -100.0 <= response[2] <= 100.0

    assert isinstance(response[0], float)
    assert isinstance(response[1], float)
    assert isinstance(response[2], float)


def test_compass():
    response = MicrobitInput.compass("A", "Compass")
    response = MicrobitInput.compass("A")

    assert 0 <= response <= 359
    assert isinstance(response, int)


def test_magnetometer():
    response = MicrobitInput.magnetometer("A")

    assert -180.0 <= response[0] <= 180.0
    assert -180.0 <= response[1] <= 180.0
    assert -180.0 <= response[2] <= 180.0

    assert isinstance(response[0], int)
    assert isinstance(response[1], int)
    assert isinstance(response[2], int)


def test_button():
    assert not MicrobitInput.button("A", "A")
    assert not MicrobitInput.button("A", "B")
    assert not MicrobitInput.button("A", "LOGO")
    assert not MicrobitInput.button("A", "Logo")
    assert not MicrobitInput.button("A", "logo")

    with pytest.raises(Exception) as e:
        MicrobitInput.button("A", "BAD")
    assert e.value.message == "Error: Request to device failed"


def test_sound():
    response = MicrobitInput.sound("A")

    assert 0 <= response <= 100
    assert isinstance(response, int)


def test_temperature():
    response = MicrobitInput.temperature("A")

    assert 0 <= response <= 50
    assert isinstance(response, int)


def test_is_shaking():
    response = MicrobitInput.is_shaking("A")

    assert not response


def test_orientation():
    response = MicrobitInput.orientation("A")

    some_position = False
    for orientation in Constant.HUMMINGBIRD_ORIENTATION_RESULTS:
        some_position = some_position or (orientation == response)

    assert some_position

def test_v2_required_temperature(mocker):
    mocker.patch.object(Request, "response", return_value="micro:bit v2 required")

    response = MicrobitInput.temperature("A")

    assert response == 0

def test_v2_required_sound(mocker):
    mocker.patch.object(Request, "response", return_value="micro:bit v2 required")

    response = MicrobitInput.sound("A")

    assert response == 0



from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1 - changing the 100 to smaller numbers makes the LED less bright
bird.setLED(1, 100)  # Turn on LED 1
sleep(1)  # Wait 1 second

bird.stopAll()  # Turn everything off

# Exercise 2
bird.setLED(1, 100)  # Turn on LED 1
sleep(5)  # Wait 5 seconds

# Exercise 3
bird.setLED(1, 100)  # Turn on LED 1
sleep(1)  # Wait 1 second
bird.setLED(1, 0)  # Turn off LED 1
bird.setLED(2, 100)  # Turn on LED 2
sleep(1)  # Wait 1 second

# Exercise 3 - modified
bird.setLED(1, 100)  # Turn on LED 1
sleep(1)  # Wait 1 second
bird.setLED(1, 0)  # Turn off LED 1
bird.setLED(2, 100)  # Turn on LED 2
sleep(1)  # Wait 1 second
bird.setLED(3, 100)  # Turn on LED 3
sleep(1)  # Wait 1 second

# Exercise 4
bird.setLED(1, 100)  # Turn on LED 1
bird.setLED(2, 100)  # Turn on LED 2
bird.setLED(3, 100)  # Turn on LED 3
sleep(2)
bird.setLED(1, 75)  # Turn on LED 1 at 75%
bird.setLED(2, 75)  # Turn on LED 2 at 75%
bird.setLED(3, 75)  # Turn on LED 3 at 75%
sleep(2)
bird.setLED(1, 50)  # Turn on LED 1 at 50%
bird.setLED(2, 50)  # Turn on LED 2 at 50%
bird.setLED(3, 50)  # Turn on LED 3 at 50%
sleep(2)
bird.setLED(1, 25)  # Turn on LED 1 at 25%
bird.setLED(2, 25)  # Turn on LED 2 at 25%
bird.setLED(3, 25)  # Turn on LED 3 at 25%
sleep(2)

bird.stopAll()  # Turn everything off



import pytest

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.hummingbird_input import HummingbirdInput


def test_acceleration():
    response = HummingbirdInput.acceleration("A")

    assert -100.0 <= response[0] <= 100.0
    assert -100.0 <= response[1] <= 100.0
    assert -100.0 <= response[2] <= 100.0

    assert isinstance(response[0], float)
    assert isinstance(response[1], float)
    assert isinstance(response[2], float)


def test_compass():
    response = HummingbirdInput.compass("A")

    assert 0 <= response <= 359
    assert isinstance(response, int)


def test_magnetometer():
    response = HummingbirdInput.magnetometer("A")

    assert -180.0 <= response[0] <= 180.0
    assert -180.0 <= response[1] <= 180.0
    assert -180.0 <= response[2] <= 180.0

    assert isinstance(response[0], int)
    assert isinstance(response[1], int)
    assert isinstance(response[2], int)


def test_orientation():
    response = HummingbirdInput.orientation("A")

    some_position = False
    for orientation in Constant.HUMMINGBIRD_ORIENTATION_RESULTS:
        some_position = some_position or (orientation == response)

    assert some_position


def test_sensor():
    response = HummingbirdInput.sensor("A", 1)

    assert isinstance(response, float)


def test_light():
    response = HummingbirdInput.light("A", 1)
    assert 0 <= response <= 100
    assert isinstance(response, int)


def test_sound():
    response = HummingbirdInput.sound("A", 1)
    assert 0 <= response <= 100
    assert isinstance(response, int)

    with pytest.raises(Exception) as e:
        response = HummingbirdInput.sound("A", 4)
    assert e.value.message == "Error: The device is not connected"


def test_sound_microbit():
    response = HummingbirdInput.sound("A", "micro:bit")

    assert 0 <= response <= 100
    assert isinstance(response, int)


def test_distance():
    response = HummingbirdInput.distance("A", 2)

    assert 0 <= response <= 298
    assert isinstance(response, int)


def test_dial():
    response = HummingbirdInput.dial("A", 1)

    assert 0 <= response <= 100
    assert isinstance(response, int)


def test_voltage():
    response = HummingbirdInput.voltage("A", 1)

    assert 0.0 <= response <= 3.3
    assert isinstance(response, float)



from birdbrain import Hummingbird, Microbit
from time import sleep

bird1 = Hummingbird('A')  # Declare Hummingbird object
bird2 = Hummingbird('B')  # Declare Microbit object

while not (bird1.getButton('A')):

    # Light sensor of the first Hummingbird controls a light on the second
    if bird1.getLight(1) < 10:
        bird2.setTriLED(1, 0, 100, 0)
    else:
        bird2.setTriLED(1, 0, 0, 0)

    # Distance sensor on the second Hummingbird controls a motor on the first
    if bird2.getDistance(1) < 20:
        bird1.setPositionServo(1, 90)
    else:
        bird1.setPositionServo(1, 0)

bird1.stopAll()
bird2.stopAll()



import time

from birdbrain.constant import Constant
from birdbrain.finch_input import FinchInput
from birdbrain.request import Request
from birdbrain.utility import Utility


class FinchOutput(Request):
    @classmethod
    def beak(self, device, r_intensity, g_intensity, b_intensity):
        """Set beak to a valid intensity. Each intensity should be an integer from 0 to 100."""
        return self.tri_led_response(device, 1, r_intensity, g_intensity, b_intensity, Constant.VALID_BEAK_PORTS)

    @classmethod
    def tail(self, device, port, r_intensity, g_intensity, b_intensity):
        """Set tail to a valid intensity. Port can be specified as 1, 2, 3, 4, or all.
        Each intensity should be an integer from 0 to 100."""

        if not port == "all":
            port = int(port) + 1  # tail starts counting at 2

        return self.tri_led_response(device, port, r_intensity, g_intensity, b_intensity, Constant.VALID_TAIL_PORTS, True)

    @classmethod
    def move(self, device, direction, distance, speed, wait_to_finish_movement=True):
        """Move the Finch forward or backward for a given distance at a given speed.
        Direction should be specified as 'F' or 'B'."""
        calc_direction = None

        if direction == Constant.FORWARD:
            calc_direction = 'Forward'
        if direction == Constant.BACKWARD:
            calc_direction = 'Backward'

        calc_distance = Utility.bounds(distance, -10000, 10000)
        calc_speed = Utility.bounds(speed, 0, 100)

        return self.__move_and_wait(
            device, wait_to_finish_movement, 'hummingbird', 'out', 'move', device, calc_direction, calc_distance, calc_speed
        )

    @classmethod
    def turn(self, device, direction, angle, speed, wait_to_finish_movement=True):
        """Turn the Finch right or left to a given angle at a given speed.
        Direction should be specified as 'R' or 'L'."""
        calc_direction = Request.calculate_left_or_right(direction)
        calc_angle = Utility.bounds(angle, 0, 360)
        calc_speed = Utility.bounds(speed, 0, 100)

        return self.__move_and_wait(
            device, wait_to_finish_movement, 'hummingbird', 'out', 'turn', device, calc_direction, calc_angle, calc_speed
        )

    @classmethod
    def wait(self, device):
        timeout_time = time.time() + Constant.MOVE_TIMEOUT_SECONDS

        while (timeout_time > time.time()) and (FinchInput.is_moving(device)):
            time.sleep(Constant.MOVE_CHECK_MOVING_DELAY)

        return True

    @classmethod
    def motors(self, device, left_speed, right_speed):
        """Set the speed of each motor individually. Speed should be in
        the range of -100 to 100."""

        left_speed = Utility.bounds(left_speed, -100, 100)
        right_speed = Utility.bounds(right_speed, -100, 100)

        return Request.response_status('hummingbird', 'out', 'wheels', device, left_speed, right_speed)

    @classmethod
    def stop(self, device):
        """Stop the Finch motors."""

        return Request.response_status('hummingbird', 'out', 'stopFinch', device)

    @classmethod
    def reset_encoders(self, device):
        """Reset both encoder values to 0."""

        response = Request.response_status('hummingbird', 'out', 'resetEncoders', device)

        time.sleep(Constant.RESET_ENCODERS_DELAY)  # finch needs a chance to actually reset

        return response

    @classmethod
    def __move_and_wait(self, device, wait_to_finish_movement, *args):
        response = Request.response_status(*args)

        time.sleep(Constant.MOVE_START_WAIT_SECONDS)  # hack to give time to start before waiting

        if wait_to_finish_movement:
            self.wait(device)

        return response



import pytest
import time

from birdbrain.exception import Exception
from birdbrain.microbit_output import MicrobitOutput
from birdbrain.request import Request
from birdbrain.state import State


def test_display():
    state = State()

    MicrobitOutput.display(state, "A", [0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0])

    time.sleep(0.15)

    Request.stop_all("A")


def test_display_wrong_size():
    with pytest.raises(Exception) as e:
        state = State()

        list = [0, 1]

        MicrobitOutput.display(state, "A", list)
    assert e.value.message == "Error: display() requires a list of length 25"


def test_point_and_clear_display():
    state = State()

    for i in range(2):
        assert MicrobitOutput.point(state, "A", 1, 1, 1)
        assert MicrobitOutput.point(state, "A", 1, 5, 1)
        assert MicrobitOutput.point(state, "A", 5, 1, 1)
        assert MicrobitOutput.point(state, "A", 5, 5, 1)

        time.sleep(0.15)

        MicrobitOutput.clear_display(state, "A")


def test_point_true_or_false():
    state = State()

    assert MicrobitOutput.point(state, "A", 3, 3, True)

    time.sleep(0.15)

    assert MicrobitOutput.point(state, "A", 3, 3, False)


def test_point_out_of_range():
    with pytest.raises(Exception) as e:
        state = State()

        assert MicrobitOutput.point(state, "A", 999, 1, 1)
    assert e.value.message == "Error: point out of range"


def test_print():
    state = State()

    assert MicrobitOutput.print(state, "A", "B")
    time.sleep(1)

    assert MicrobitOutput.print(state, "A", " ")
    time.sleep(1)


def test_print_nothing():
    state = State()

    assert MicrobitOutput.print(state, "A", "")
    time.sleep(1)

    assert MicrobitOutput.print(state, "A", None)
    time.sleep(1)


def test_play_note():
    assert MicrobitOutput.play_note("A", 50, 0.25)



import pytest
import time

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.finch import Finch
from birdbrain.hummingbird import Hummingbird
from birdbrain.request import Request


def test_is():
    finch = Finch.connect("B")

    assert finch.is_connected()
    assert finch.is_microbit()
    assert not finch.is_hummingbird()
    assert finch.is_finch()

    assert finch.isConnectionValid()
    assert finch.isMicrobit()
    assert not finch.isHummingbird()
    assert finch.isFinch()

    with pytest.raises(Exception) as e:
        hummingbird = Hummingbird('B')
    assert e.value.message == "Error: Device B is not a Hummingbird"


def test_beak_with_alias():
    finch = Finch('B')

    finch.beak(100, 50, 50)
    time.sleep(0.15)
    finch.setBeak(0, 0, 0)


def test_tail_with_alias():
    finch = Finch("B")

    assert finch.tail(1, 10, 0, 50)
    time.sleep(0.1)
    assert finch.tail(1, "50", 0, "0")
    time.sleep(0.1)
    assert finch.tail("2", "50", 0, "0")
    time.sleep(0.1)
    assert finch.tail(3, "50", 0, "0")
    time.sleep(0.1)
    assert finch.tail(4, "50", 0, "0")
    time.sleep(0.1)
    assert finch.tail("all", 100, 0, 100)
    time.sleep(0.1)
    assert finch.setTail("all", 0, 0, 0)


def test_move_with_alias():
    finch = Finch("B")

    assert finch.move(Constant.FORWARD, 4, 20)
    assert finch.move(Constant.FORWARD, "4", "20")
    assert finch.move(Constant.BACKWARD, 4, 20)
    assert finch.setMove(Constant.BACKWARD, "4", "20")

    with pytest.raises(Exception) as e:
        finch = Finch("B")

        assert finch.move(None, 4, 20)

    assert e.value.message == "Error: Request to device failed"

    with pytest.raises(Exception) as e:
        finch = Finch("B")

        assert finch.move("BAD", 4, 20)

    assert e.value.message == "Error: Request to device failed"


def test_is_moving():
    finch = Finch("B")

    assert finch.move(Constant.FORWARD, 7, 5, False)
    assert finch.is_moving()

    assert finch.wait("B")

    assert finch.move(Constant.BACKWARD, 7, 5, True)

    finch.stop_all()

    time.sleep(1)

    assert not finch.is_moving()


def test_turn_with_alias():
    finch = Finch("B")

    finch.turn("L", 45, 50)
    finch.turn("R", 45, 50)
    finch.turn("L", "45", 50)
    finch.setTurn("R", 45, "50")


def test_motors_with_alias():
    finch = Finch("B")

    assert finch.motors(25, 0)
    time.sleep(0.2)
    assert finch.motors(-25, 0)
    time.sleep(0.2)

    assert finch.motors(0, -25)
    time.sleep(0.2)
    assert finch.motors("0", "25")
    time.sleep(0.2)

    Request.stop_all("B")

    Request.stop_all("B")


def test_stop():
    finch = Finch("B")

    assert finch.move(Constant.FORWARD, 99999, 5, False)
    time.sleep(0.2)
    assert finch.stop()

    assert finch.move(Constant.BACKWARD, 99999, 5, False)
    time.sleep(0.2)
    assert finch.stop()


def test_reset_encoders():
    finch = Finch("B")

    assert finch.reset_encoders()
    assert finch.resetEncoders()


def test_light_with_alias():
    finch = Finch("B")

    assert 0 <= finch.light("L") <= 100
    assert isinstance(finch.getLight("L"), int)

    assert 0 <= finch.light("R") <= 100
    assert isinstance(finch.getLight("R"), int)

    with pytest.raises(Exception) as e:
        finch.light("BAD")
    assert e.value.message == "Error: Request to device failed"


def test_distance_with_alias():
    finch = Finch("B")

    response = finch.distance()
    response = finch.getDistance()

    assert 0 <= response <= 298
    assert isinstance(response, int)


def test_line_with_alias():
    finch = Finch("B")

    assert 0 <= finch.line("L") <= 100
    assert isinstance(finch.getLine("L"), int)

    assert 0 <= finch.line("R") <= 100
    assert isinstance(finch.getLine("R"), int)

    with pytest.raises(Exception) as e:
        finch.line("BAD")
    assert e.value.message == "Error: Request to device failed"


def test_encoder_with_alias():
    finch = Finch("B")

    assert -100.0 <= finch.encoder("L") <= 100.0
    assert isinstance(finch.getEncoder("L"), float)

    assert -100.0 <= finch.encoder("R") <= 100.0
    assert isinstance(finch.getEncoder("R"), float)

    with pytest.raises(Exception) as e:
        finch.encoder("BAD")
    assert e.value.message == "Error: Request to device failed"


def test_orientation_with_alias():
    finch = Finch("B")

    response = finch.orientation()
    response = finch.getOrientation()

    some_position = False
    for orientation in Constant.FINCH_ORIENTATION_RESULTS:
        some_position = some_position or (orientation == response)

    assert some_position


def test_stop_all():
    finch = Finch("B")

    finch.stop_all()
    finch.stopAll()



import pytest

from birdbrain.exception import Exception
from birdbrain.finch import Finch
from birdbrain.hummingbird import Hummingbird


def helper_test_acceleration(device):
    response = device.acceleration()
    response = device.getAcceleration()

    assert -100.0 <= response[0] <= 100.0
    assert -100.0 <= response[1] <= 100.0
    assert -100.0 <= response[2] <= 100.0

    assert isinstance(response[0], float)
    assert isinstance(response[1], float)
    assert isinstance(response[2], float)


def helper_test_compass(device):
    response = device.compass()
    response = device.getCompass()

    assert 0 <= response <= 359
    assert isinstance(response, int)


def helper_test_magnetometer(device):
    response = device.magnetometer()
    response = device.getMagnetometer()

    assert -100 <= response[0] <= 100
    assert -100 <= response[1] <= 100
    assert -100 <= response[2] <= 100

    assert isinstance(response[0], int)
    assert isinstance(response[1], int)
    assert isinstance(response[2], int)


def helper_test_button(device):
    assert not device.button("A")
    assert not device.button("B")
    assert not device.button("LOGO")
    assert not device.button("Logo")
    assert not device.getButton("logo")

    with pytest.raises(Exception) as e:
        device.button("BAD")
    assert e.value.message == "Error: Request to device failed"


def helper_test_sound(device):
    response = device.sound(3)
    response = device.getSound(3)

    assert 0 <= response <= 100


def helper_test_temperature(device):
    response = device.temperature()
    response = device.getTemperature()

    assert 0 <= response <= 50


def helper_test_is_shaking(device):
    response = device.is_shaking()
    response = device.isShaking()

    assert not response


def helper_test_shared(device):
    helper_test_acceleration(device)
    helper_test_compass(device)
    helper_test_magnetometer(device)
    helper_test_button(device)
    helper_test_sound(device)
    helper_test_temperature(device)
    helper_test_is_shaking(device)


def test_shared():
    helper_test_shared(Hummingbird("A"))
    helper_test_shared(Finch("B"))



import pytest
import time

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.finch_output import FinchOutput
from birdbrain.request import Request


def test_beak():
    assert FinchOutput.beak("B", 10, 50, 50)
    time.sleep(0.15)
    assert FinchOutput.beak("B", 0, 0, 0)


def test_tail():
    assert FinchOutput.tail("B", 1, 10, 50, 50)
    time.sleep(0.1)
    assert FinchOutput.tail("B", 1, "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", "2", "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", 3, "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", 4, "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", "all", 0, 0, 100)
    time.sleep(0.1)
    assert FinchOutput.tail("B", "all", 0, 0, 0)


def test_move():
    assert FinchOutput.move("B", Constant.FORWARD, 4, 5)
    assert FinchOutput.move("B", Constant.FORWARD, "4", "5")

    assert FinchOutput.move("B", Constant.BACKWARD, 4, 5)
    assert FinchOutput.move("B", Constant.BACKWARD, "4", "5")

    with pytest.raises(Exception) as e:
        assert FinchOutput.move("B", "BAD", 4, 5)
    assert e.value.message == "Error: Request to device failed"

    with pytest.raises(Exception) as e:
        assert FinchOutput.move("B", None, 4, 5)
    assert e.value.message == "Error: Request to device failed"

    Request.stop_all("B")


def test_turn():
    assert FinchOutput.turn("B", "L", 25, 50)
    assert FinchOutput.turn("B", "R", 25, 50)
    assert FinchOutput.turn("B", "L", "25", 50)
    assert FinchOutput.turn("B", "R", 25, "50")

    with pytest.raises(Exception) as e:
        assert FinchOutput.turn("B", "BAD", 90, 50)
    assert e.value.message == "Error: Request to device failed"


def test_motors():
    assert FinchOutput.motors("B", 25, 0)
    time.sleep(0.2)
    assert FinchOutput.motors("B", -25, 0)
    time.sleep(0.2)

    assert FinchOutput.motors("B", 0, -25)
    time.sleep(0.2)
    assert FinchOutput.motors("B", "0", "25")
    time.sleep(0.2)

    Request.stop_all("B")


def test_stop():
    assert FinchOutput.move("B", Constant.FORWARD, 99999, 5, False)
    time.sleep(0.2)
    assert FinchOutput.stop("B")

    assert FinchOutput.move("B", Constant.BACKWARD, 99999, 5, False)
    time.sleep(0.2)
    assert FinchOutput.stop("B")


def test_reset_encoders():
    assert FinchOutput.reset_encoders("B")



from birdbrain.constant import Constant
from birdbrain.finch import Finch
from birdbrain.hummingbird import Hummingbird
from birdbrain.tasks import Tasks


# package name "BirdBrain" no longer supported...use "birdbrain" instead


def test_instantiating_devices_old_way():
    from birdbrain import Constant, Finch, Hummingbird, Tasks

    Constant()
    Finch('B')
    Hummingbird('A')
    Tasks()

    Constant()
    Finch('B')
    Hummingbird('A')
    Tasks()

    assert Constant().LEFT == 'L'




import pytest
import time

from birdbrain.constant import Constant
from birdbrain.exception import Exception
from birdbrain.finch import Finch
from birdbrain.hummingbird import Hummingbird


def test_connect_device_name_as_none():
    with pytest.raises(Exception) as e:
        Hummingbird(None)
    assert e.value.message == "Missing device name"


def test_connect_bad_device_name():
    with pytest.raises(Exception) as e:
        Hummingbird('D')
    assert e.value.message == "Invalid device name: D"


def test_connect_valid_device_name():
    hummingbird = Hummingbird("A")

    assert hummingbird.device == "A"


def test_is():
    hummingbird = Hummingbird("A")

    assert hummingbird.is_connected()
    assert hummingbird.is_microbit()
    assert hummingbird.is_hummingbird()
    assert not hummingbird.is_finch()

    assert hummingbird.isConnectionValid()
    assert hummingbird.isMicrobit()
    assert hummingbird.isHummingbird()
    assert not hummingbird.isFinch()

    with pytest.raises(Exception) as e:
        finch = Finch('A')
    assert e.value.message == "Error: Device A is not a Finch"


def test_led_with_alias():
    hummingbird = Hummingbird("A")

    assert hummingbird.led(1, 100)
    time.sleep(0.15)

    assert hummingbird.led(1, 0)
    time.sleep(0.15)

    assert hummingbird.led(1, 50)
    time.sleep(0.15)

    hummingbird.setLED(1, 0)


def test_led_no_connection():
    with pytest.raises(Exception) as e:
        hummingbird = Hummingbird('C')
    assert e.value.message == "No connection: C"


def test_tri_led_with_alias():
    hummingbird = Hummingbird("A")

    assert hummingbird.tri_led(1, 50, "50", 0)
    time.sleep(0.15)

    assert hummingbird.setTriLED(1, 100, "0", "0")
    time.sleep(0.15)

    assert hummingbird.tri_led(1, 0, "0", "0")
    time.sleep(0.15)


def test_position_servo_with_alias():
    hummingbird = Hummingbird("A")

    assert hummingbird.position_servo(1, 50)
    time.sleep(0.15)

    assert hummingbird.setPositionServo(1, "130")


def test_rotation_servo_with_alias():
    hummingbird = Hummingbird("A")

    assert hummingbird.rotation_servo(2, 50)
    time.sleep(0.15)

    assert hummingbird.setRotationServo(2, "-50")
    time.sleep(0.15)

    assert hummingbird.rotation_servo(2, 100)
    time.sleep(0.15)

    assert hummingbird.setRotationServo(2, -100)
    time.sleep(0.15)

    assert hummingbird.setRotationServo(2, 0)


def test_orientation_with_alias():
    hummingbird = Hummingbird("A")

    response = hummingbird.orientation()
    response = hummingbird.getOrientation()

    some_position = False
    for orientation in Constant.HUMMINGBIRD_ORIENTATION_RESULTS:
        some_position = some_position or (orientation == response)

    assert some_position


def test_sensor():
    hummingbird = Hummingbird("A")

    response = hummingbird.sensor(1)
    response = hummingbird.getSensor(1)

    assert isinstance(response, float)


def test_light_with_alias():
    hummingbird = Hummingbird("A")

    response = hummingbird.light(3)
    response = hummingbird.getLight(3)
    response = hummingbird.light("3")

    assert 0 <= response <= 100
    assert isinstance(response, int)

    with pytest.raises(Exception) as e:
        response = hummingbird.light(4)
    assert e.value.message == "Error: The device is not connected"


def test_sound_with_alias():
    hummingbird = Hummingbird("A")

    response = hummingbird.sound(3)
    response = hummingbird.getSound(3)
    response = hummingbird.sound("3")

    assert 0 <= response <= 100
    assert isinstance(response, int)

    with pytest.raises(Exception) as e:
        response = hummingbird.sound(4)
    assert e.value.message == "Error: The device is not connected"


def test_sound_microbit():
    hummingbird = Hummingbird("A")

    response = hummingbird.sound("micro:bit")

    assert 0 <= response <= 100
    assert isinstance(response, int)


def test_distance_with_alias():
    hummingbird = Hummingbird("A")

    response = hummingbird.distance(2)
    response = hummingbird.getDistance(2)
    response = hummingbird.distance("2")

    assert 0 <= response <= 298
    assert isinstance(response, int)


def test_dial_with_alias():
    hummingbird = Hummingbird("A")

    response = hummingbird.dial(1)
    response = hummingbird.getDial(1)
    response = hummingbird.dial("1")

    assert 0 <= response <= 100
    assert isinstance(response, int)


def test_voltage_with_alias():
    hummingbird = Hummingbird("A")

    response = hummingbird.voltage(1)
    response = hummingbird.getVoltage(1)

    assert isinstance(response, float)


def test_stop_all():
    hummingbird = Hummingbird("A")

    hummingbird.stop_all()
    hummingbird.stopAll()



# Note: For simplicity, we focused only on if-else statements. Some challenges
# could be completed with if statements that don't have an else.

from birdbrain import Hummingbird
from time import sleep

bird = Hummingbird('A')  # Declare Hummingbird object

# Exercise 1
print(bird.getLight(1))  # Print sensor value

# Exercise 2
if bird.getLight(1) < 10:  # if it is dark
    bird.setLED(1, 100)  # turn on the light
    bird.setDisplay([0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0])  # Display <
else:  # otherwise
    bird.setLED(1, 0)  # turn off the light
    bird.setDisplay([0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0])  # Display >
sleep(1)

# Exercise 3 - when the threshold is 0, the light will never come on (because
# the light sensor can't have a value less than 0). When the threshold is 101, the
# light will always come on (because the light sensor value is always less than 101).

threshold = 10  # declaring a variable
if bird.getLight(1) < threshold:  # if it is dark
    bird.setLED(1, 100)  # turn on the light
else:  # otherwise
    bird.setLED(1, 0)  # turn off the light
sleep(5)

# Exercise 4
# Note that we don't need to declare the variable again
if bird.getLight(1) > threshold:  # if it is bright
    bird.setRotationServo(1, 100)  # turn on the motor
else:  # otherwise
    bird.setRotationServo(1, 0)  # turn off the motor
sleep(5)
bird.setRotationServo(1, 0)  # turn it off if it's on

# Exercise 5
if bird.getLight(1) < threshold:  # if it is dark
    bird.playNote(40, 1)  # low note
else:  # otherwise
    bird.playNote(80, 1)  # high note
sleep(1)

# Extra Challenge
for i in range(25):
    if bird.getLight(1) < threshold:  # if it is dark
        bird.playNote(40, 1)  # low note
    else:  # otherwise
        bird.playNote(80, 1)  # high note
    sleep(1)

bird.stopAll()

# micro:bit V2 Extra Challenge
if bird.getTemperature() > 20:  # if it is hot
    bird.setTriLED(1, 100, 0, 0)  # red
else:  # otherwise
    bird.setTriLED(1, 0, 0, 100)  # blue
sleep(1)



from birdbrain.exception import Exception
from birdbrain.hummingbird_input import HummingbirdInput
from birdbrain.hummingbird_output import HummingbirdOutput
from birdbrain.microbit import Microbit
from birdbrain.request import Request


class Hummingbird(Microbit):
    """Hummingbird Bit Class includes the control of the outputs and inputs
    present on the Hummingbird Bit."""

    def __init__(self, device='A', raise_exception_if_no_connection=True):
        self.device_object = Hummingbird.connect(device, raise_exception_if_no_connection)

        if not self.device_object.is_hummingbird():
            raise Exception("Error: Device " + device + " is not a Hummingbird")

    def led(self, port, intensity):
        return HummingbirdOutput.led(self.device, port, intensity)

    def tri_led(self, port, r_int, g_int, b_int):
        return HummingbirdOutput.tri_led(self.device, port, r_int, g_int, b_int)

    def position_servo(self, port, angle):
        return HummingbirdOutput.position_servo(self.device, port, angle)

    def rotation_servo(self, port, speed):
        return HummingbirdOutput.rotation_servo(self.device, port, speed)

    def sensor(self, port):
        return HummingbirdInput.sensor(self.device, port)

    def light(self, port):
        return HummingbirdInput.light(self.device, port)

    def sound(self, port):
        return HummingbirdInput.sound(self.device, port)

    def distance(self, port):
        return HummingbirdInput.distance(self.device, port)

    def dial(self, port):
        return HummingbirdInput.dial(self.device, port)

    def voltage(self, port):
        return HummingbirdInput.voltage(self.device, port)

    def stop_all(self):
        Request.stop_all(self.device)

    getDial = dial
    getDistance = distance
    setLED = led
    getLight = light
    setPositionServo = position_servo
    setRotationServo = rotation_servo
    getSound = sound
    getSensor = sensor
    stopAll = stop_all
    setTriLED = tri_led
    getVoltage = voltage



from birdbrain.constant import Constant
from birdbrain.request import Request
from birdbrain.utility import Utility


class HummingbirdOutput(Request):
    @classmethod
    def led(self, device, port, intensity):
        """Set led  of a certain port requested to a valid intensity."""
        self.validate_port(port, Constant.VALID_LED_PORTS)

        calculated_intensity = Utility.bounds(Request.calculate_intensity(intensity), 0, 255)

        return Request.response_status('hummingbird', 'out', 'led', port, calculated_intensity, device)

    @classmethod
    def tri_led(self, device, port, r_intensity, g_intensity, b_intensity):
        """Set TriLED  of a certain port requested to a valid intensity."""
        return self.tri_led_response(device, port, r_intensity, g_intensity, b_intensity, Constant.VALID_TRI_LED_PORTS)

    @classmethod
    def position_servo(self, device, port, angle):
        """Set Position servo of a certain port requested to a valid angle."""
        Request.validate_port(port, Constant.VALID_SERVO_PORTS)

        calculated_angle = Utility.bounds(Request.calculate_angle(angle), 0, 254)

        return Request.response_status('hummingbird', 'out', 'servo', port, calculated_angle, device)

    @classmethod
    def rotation_servo(self, device, port, speed):
        """Set Rotation servo of a certain port requested to a valid speed."""
        Request.validate_port(port, Constant.VALID_SERVO_PORTS)

        calculated_speed = Request.calculate_speed(Utility.bounds(int(speed), -100, 100))

        return Request.response_status('hummingbird', 'out', 'rotation', port, calculated_speed, device)
